-------------------------------------------------------------------------------
-- Title      : LedCtrlSlink
-- Project    : 
-------------------------------------------------------------------------------
-- File       : LedCtrlSlink.vhd
-- Author     :   <otger@OTGER-PC>
-- Company    : 
-- Created    : 2010-12-14
-- Last update: 2010-12-15
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: Simple controller to enable disable slink leds for MCB
--              After a reset, leds flash for 25ms
--              On each rising edge of the signals, its led flash
-------------------------------------------------------------------------------
-- Copyright (c) 2010 
-------------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- sysClk       : System Clock
-- reset        : Resets all Registers
-- sel          : select this module for read/write_n
-- rdWr_n       : Read or write_n when sel is set
-- activityLedSrc: ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH and led enabled
-- linkUpLedSrc : ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH and led enabled
-- xoffLedSrc   : ONE SHOT FLASH WHEN THIS SIGNAL GOES HIGH and led enabled
-- dataIn       : DATA TO SELECT WHICH STIMULUS IS ACTIVE
-- dataOut      : READ BACK FOR PAN 
-- activityLed  : actual led signal
-- linkUpLed    : actual led signal
-- xoffLed      : actual led signal
-- 
------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2010-12-14  1.0      otger   Created
-----------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity LedCtrlSlink is
  port (
    sysClk         : in  std_logic;
    reset          : in  std_logic;
    sel            : in  std_logic;
    rdWr_n         : in  std_logic;
    dataIn         : in  std_logic_vector(2 downto 0);
    dataOut        : out std_logic_vector(15 downto 0);
    activityLedSrc : in  std_logic;
    linkUpLedSrc   : in  std_logic;
    xoffLedSrc     : in  std_logic;
    activityLed    : out std_logic;
    linkUpLed      : out std_logic;
    xoffLed        : out std_logic
    );
end LedCtrlSlink;

architecture Behavioral of LedCtrlSlink is

  -- 25 ms pulse width.
  constant LED_PERIOD : std_logic_vector(19 downto 0) := x"F4240";

  signal ledEnReg   : std_logic_vector(2 downto 0);
  signal srcReg     : std_logic_vector(2 downto 0);
  signal risingEdge : std_logic_vector(2 downto 0);
  signal ledFlash   : std_logic_vector(2 downto 0);
  signal firstRun   : std_logic_vector(2 downto 0);

  type nineteen is array (0 to 2) of std_logic_vector(19 downto 0);
  signal counter : nineteen;
begin

------------------------------------------------------------------------
-- Ya' gotta catch the data value to set the the bits baby!
------------------------------------------------------------------------
  enableRegister : process (sysClk, reset)
  begin  -- process enableRegister
    if reset = '1' then                 -- asynchronous reset (active high)
      ledEnReg <= "000";
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if sel = '1' and rdWr_n = '0' then
        ledEnReg(2 downto 0) <= dataIn(2 downto 0);
      end if;
    end if;
  end process enableRegister;

  registeringSources : process (sysClk, reset)
  begin  -- process registeringSources
    if reset = '1' then                 -- asynchronous reset (active high)
      srcReg <= "111";   -- To avoid detecting false rising up after reset
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      srcReg(0) <= activityLedSrc;
      srcReg(1) <= xoffLedSrc;
      srcReg(2) <= linkUpLedSrc;
    end if;
  end process registeringSources;

  risingEdge(0) <= activityLedSrc xor srcReg(0);
  risingEdge(1) <= xoffLedSrc xor srcReg(1);
  risingEdge(2) <= linkUpLedSrc xor srcReg(2);

  flashGenerator : for i in 0 to 2 generate
    counterReg : process (sysClk, reset)
    begin  -- process counterReg
      if reset = '1' then               -- asynchronous reset (active high)
        counter(i)  <= (others => '0');  -- This triggers a flash after reset
        ledFlash(i) <= '0';
        firstRun(i) <= '1';
      elsif sysClk'event and sysClk = '1' then  -- rising clock edge
        if risingEdge(i) = '1' then
          counter(i)  <= (others => '0');
          ledFlash(i) <= '1';
        else
          if counter(i) < LED_PERIOD then
            counter(i) <= counter(i) + 1;
          else
            ledFlash(i) <= '0';
            firstRun(i) <= '0';
          end if;
        end if;
      end if;
    end process counterReg;
  end generate flashGenerator;

  activityLed <= (ledFlash(0) or activityLedSrc) and
                 (ledEnReg(0) or firstRun(0));
  xoffLed     <= (ledFlash(1) or xoffLedSrc) and 
                 (ledEnReg(1) or firstRun(1));
  linkUpLed   <= (ledFlash(2) or linkUpLedSrc) and 
                 (ledEnReg(2) or firstRun(2));

  dataOut <= x"000" & '0' & LedEnReg;
  
end Behavioral;
