------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U20
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   SEQUENCER COMMAND DECODER
-- VERSION      :   4.5 
------------------------------------------------------------------------
-- FILE NAME    :   PIXCMDDECODEV45.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   10/23/2005
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :  RECEIVES 32 BIT WORDS FROM VIA THE FPGA INTERFACE, DECODES
-- THE COMMAND AND ASSEMBLES THE DATA STRUCTURE TO CONTROL THE INNNER FUNCTIONS
-- OF THE PIXEL FPGA. WHERE AS THE PAN DECODE MODULE IN THE SEQUENCER FPGA NEEDS
-- TO DEAL WITH ONE AND TWO WORD COMMANDS, THIS MODULE ONLY DECODES AND SEQUENCES
-- SIMPLE SINGLE WORD COMMANDS, I.E 32 BIT READS, 16 BIT WRITES. 
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 04/17/2004  PCM   4.0  ORIGINAL VERSION
-- 08/14/2004  pcm   4.1  added select strobes for fpdp word counter registers.
-- 02/14/2005  PCM   4.4  ADDED CONTROL REGISTER CMD STROBE + DEBUG REGISTER SUPPORT.
-- 10/23/2005  PCM   4.5  ADDED FIRMWARE IDENTITY REGISTER + DATA_WIDTH SIGNAL FOR MUX
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : SYSTEM CLOCK
-- RESET        : SYSTEM RESET
-- CMD_STROBE   : CMD_DATA FROM THE FPGA INTERFACE IS VALID
-- CMD_DATA     : 32 BIT COMMAND WORD
-- CFG_CMD_STB  : COMMAND STROBE TO SERIAL CONFIGURATION MODULE
-- CFG_ADDRESS  : ADDRESS OF CONFIGURATION MEMORY TO READ
-- ADC_CMD_STB  : COMMAND STROBE TO READ AN ADC CHANNEL
-- DAC_CMD_STB  : COMMAND STROBE TO WRITE A DAC VALUE
-- ADCDAC_DATA  : DATA OUTPUT TO DAC CHANNELS
-- ADCDAC_ADDR  : ADDRESS OF DAC CHANNEL TO UPDATE
-- EXD_CMD_STB  : COMMAND STROBE TO WRITE AN EXTERNAL DATA NIBBLE
-- EX_DATA      : DATA OUT TO EXTERNAL DATA LATCH.
-- CLK_CMD_STB  : COMMAND STROBE TO SET THE CLOCK GENERATOR PHASE BITS
-- CLK_DATA     : DATA OUT TO CLOCK PHASE LATCH
-- FPDP_CMD_STB : CMD STROBE TO ACCESS THE DEBUG REGISTER SET IN THE FPDP MODULE
-- CNTL_CMD_STB : CMD STROBE TO THE LOCAL (PIXEL FPGA) CONTROL REGISTER
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

use work.constantsPkg.all;

entity PixCmdDecodeV461 is
  port (SYSCLK        : in    std_logic;
        RESET         : in    std_logic;
        CMD_RDY       : in    std_logic;
        CMD_READ      : in    std_logic;
        CMD_DATA      : in    std_logic_vector(31 downto 0);
        ADDRESS       : out   std_logic_vector(15 downto 0);
        REG_DATA      : inout std_logic_vector(15 downto 0);
        DATA_OUT      : out   std_logic_vector(15 downto 0);
        READ_REQ      : out   std_logic;
        READ_ACK      : in    std_logic;
        READ_SYNC     : out   std_logic;
        READ_WRITE_N  : out   std_logic;
--        CFG_CMD_STB   : out   std_logic;
        EXD_CMD_STB   : out   std_logic;
--        FCH_CMD_STB   : out   std_logic;
        LED_CMD_STB   : out   std_logic;
        SLINK_CMD_STB : out   std_logic;
        DBL_WRD_XFER  : out   std_logic;
        SIM_ENABLE    : out   std_logic;
        SIM_PIXROWS   : out   std_logic_vector(15 downto 0);
        SIM_PIXCOLS   : out   std_logic_vector(15 downto 0);
        SIM_PIXTIME   : out   std_logic_vector(7 downto 0);
        SIM_CHANNELS  : out   std_logic_vector(6 downto 0);
        SIM_DELAY     : out   std_logic_vector(7 downto 0)
        );
end PixCmdDecodeV461;

architecture Behavioral of PixCmdDecodeV461 is


  function csGenerator (
    constant regAddr : std_logic_vector;
    signal currAddr  : std_logic_vector)
    return std_logic is
    variable tmp : std_logic := '1';
  begin
    for i in regAddr'range loop
      if regAddr(i) = '1' then
        tmp := tmp and currAddr(i);
      else
        tmp := tmp and not currAddr(i);
      end if;
    end loop;  -- i
    return tmp;
  end csGenerator;

  signal tmp_readWrite_n : std_logic;
  signal tmpAddress      : std_logic_vector(15 downto 0);

  type PixCmdDecodeState is (PixIdle, PixWrt1, PixRd1, PixRd2, PixRd3);
  signal PixState     : PixCmdDecodeState;
  signal LocalData    : std_logic_vector(31 downto 0);
--  signal SerCfg_cs    : std_logic;
  signal LedPrt_cs    : std_logic;
--  signal Fetch_cs     : std_logic;
  signal ExtDat_cs    : std_logic;
  signal firmVers_cs  : std_logic;
  signal pixCtlReg_cs : std_logic;

  signal pixRowReg_cs  : std_logic;
  signal pixColReg_cs  : std_logic;
  signal pixTimeReg_cs : std_logic;
  signal pixChanReg_cs : std_logic;
  signal pixDlyReg_cs  : std_logic;

  signal PixControlReg : std_logic_vector(15 downto 0);
  signal PixRowReg     : std_logic_vector(15 downto 0);
  signal PixColReg     : std_logic_vector(15 downto 0);
  signal PixTimeReg    : std_logic_vector(7 downto 0);
  signal PixChanReg    : std_logic_vector(6 downto 0);
  signal PixDlyReg     : std_logic_vector(7 downto 0);

  
begin

  READ_WRITE_N <= tmp_readWrite_n;
------------------------------------------------------------------------
-- COMMAND DECODE PROCESS
------------------------------------------------------------------------
  PixCmdDecode : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (RESET = '1') then
        PixState <= PixIdle;
        READ_REQ <= '0';
      else
        case PixState is
          when PixIdle =>
            LocalData <= CMD_DATA;
            if (CMD_RDY = '1') then
              if (CMD_READ = '1') then
                TMP_READWRITE_N <= '1';  -- Read operation required
                PixState        <= PixRd1;
              else
                TMP_READWRITE_N <= '0';  -- Write operation required
                PixState        <= PixWrt1;
              end if;
            else
              PixState <= PixIdle;
            end if;
          when PixRd1 =>
            PixState <= PixRd2;
          when PixRd2 =>
            READ_REQ <= '1';  -- Ask for service to the systran board for next
            -- clock edge
            PixState <= PixRd3;
          when PixRd3 =>
            if (READ_ACK = '1') then  -- Wait here until the data is accepted.
              READ_REQ <= '0';
              PixState <= PixIdle;
            else
              PixState <= PixRd3;
            end if;
          when PixWrt1 =>
            PixState <= PixIdle;        -- Go back to look for more commands
          when others =>
            PixState <= PixIdle;
        end case;
      end if;
    end if;
  end process;


  firmVers_cs   <= csGenerator(ADDR_FWVERHEX, tmpAddress);
  pixCtlReg_cs  <= csGenerator(ADDR_PIXCTLREG, tmpAddress);
  pixDlyReg_cs  <= csGenerator(ADDR_PIXDLYREG, tmpAddress);
  pixChanReg_cs <= csGenerator(ADDR_PIXCHANREG, tmpAddress);
  pixTimeReg_cs <= csGenerator(ADDR_PIXTIMEREG, tmpAddress);
  pixColReg_cs  <= csGenerator(ADDR_PIXCOLREG, tmpAddress);
  pixRowReg_cs  <= csGenerator(ADDR_PIXROWREG, tmpAddress);
  ExtDat_cs     <= csGenerator(ADDR_EXTDATALATCH, tmpAddress);
  LedPrt_cs     <= csGenerator(ADDR_LEDLATCH, tmpAddress);
--  Fetch_cs <= csGenerator(ADDR_SERFETCH, tmpAddress);
--  SerCfg_cs <= csGenerator(ADDR_RANGE_SCONF, tmpAddress);
  SLINK_CMD_STB <= csGenerator(ADDR_RANGE_SLINK, tmpAddress);

------------------------------------------------------------------------
-- DATA, ADDRESS, AND CHIP SELECT DEMUX.
------------------------------------------------------------------------
  DATA_OUT    <= LocalData(15 downto 0);
  tmpAddress  <= LocalData(31 downto 16);
  ADDRESS     <= tmpAddress;
--  CFG_CMD_STB <= SerCfg_cs;
  EXD_CMD_STB <= ExtDat_cs;
--  CLK_CMD_STB <= ClkCtl_cs;
--  FCH_CMD_STB <= Fetch_cs;
  LED_CMD_STB <= LedPrt_cs;

  READ_SYNC <= '1';                     -- Until further notice !

------------------------------------------------------------------------
-- Control and simulation registers + trigger functions
------------------------------------------------------------------------
  PixControlRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixCtlReg_cs = '1') then
        PixControlReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;

  DBL_WRD_XFER <= PixControlReg(0);
  SIM_ENABLE   <= PixControlReg(15);

  PixRowRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixRowReg_cs = '1') then
        PixRowReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXROWS <= PixRowReg;

  PixColRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixColReg_cs = '1') then
        PixColReg <= LocalData(15 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXCOLS <= PixColReg;

  PixTimeRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixTimeReg_cs = '1') then
        PixTimeReg <= LocalData(7 downto 0);
      end if;
    end if;
  end process;
  SIM_PIXTIME <= PixTimeReg;

  PixChanRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixChanReg_cs = '1') then
        PixChanReg <= LocalData(7 downto 1);
      end if;
    end if;
  end process;
  SIM_CHANNELS <= PixChanReg;

  PixDlyRegProc : process (SYSCLK)
  begin
    if (rising_edge(SYSCLK)) then
      if (TMP_READWRITE_N = '0' and pixDlyReg_cs = '1') then
        PixDlyReg <= LocalData(7 downto 0);
      end if;
    end if;
  end process;
  SIM_DELAY <= PixDlyReg;


  REG_DATA <=
    PixControlReg            when pixCtlReg_cs = '1' else
    PixRowReg                when pixRowReg_cs = '1' else
    PixColReg                when pixColReg_cs = '1' else
    x"00" & PixTimeReg       when pixTimeReg_cs = '1' else
    x"00" & '1' & PixChanReg when pixChanReg_cs = '1' else
    x"00" & PixDlyReg        when pixDlyReg_cs = '1' else
    FWVERHEX                 when firmVers_cs = '1' else
    x"DEFA";

end Behavioral;
