------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U20
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   PIXEL DATA MULTIPLEXOR
-- VERSION      :   4.0
------------------------------------------------------------------------
-- FILE NAME    :   PIXDATAMUXV40.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/12/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- PROVIDES A MULTIPLE SOURCE MUX FOR DATA FROM THE VARIOUS SUB MODULES 
-- IN THE PIXEL FPGA. THIS DATA IS MULTIPLEXED TO THE SYSTRAN RM MODULE.
-- A SIMPLE PRIORITY SCHEME ASSURES PRIORITY FOR PIXEL DATA, THEN SEQUENCER
-- FPGA DATA, THEN LOCAL DATA.  
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
--    DATE    AUTHOR VERSION  MODIFICATIONS
-- 02/19/2003  PCM     2.1    ORIGINAL VERSION
-- 04/12/2004  PCM     4.0    MODIFIED FOR REVISION A HARDWARE
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- PIX_DATA_IN  : PIXEL BUS DATA
-- PIX_DATA_REQ : PIXEL BUS DATA REQUEST
-- PIX_DATA_ACK : PIXEL BUS DATA ACKNOWLEDGE
-- PIX_DATA_SYNC: PIXEL DATA BUS SYNC SIGNAL
-- SEQ_DATA_IN  : SEQ FPGA INTERFACE DATA
-- SEQ_DATA_REQ : SEQ FPGA INTERFACE DATA REQUEST
-- SEQ_DATA_ACK : SEQ FPGA INTERFACE DATA ACKNOWLEDGE
-- SEQ_DATA_SYNC: INPUT FROM SEQ DATA PORT SYNC SIGNAL
-- LOC_DATA_IN  : PIX FPGA DATA
-- LOC_DATA_REQ : PIX FPGA DATA REQUEST
-- LOC_DATA_ACK : PIX FPGA DATA ACKNOWLEDGE
-- LOC_DATA_SYNC: INPUT FROM PIX FPGA SYNC SIGNAL
-- DATA_OUT     : MULTIPLEXED DATA BUS OUTPUT TO FPDP_RM 
-- DATA_WR      : WRITE STROBE TO FPDP_RM
-- DATA_BUSY    : BUSY SIGNAL FROM FPDP_RM
-- DATA_SYNC    : SYNCRONIZED PIX_DATA OR SEQ_DATA SYNC SIGNAL TO 
--                FPDP_TM SYNC SIG.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity PixDataMuxV40 is
    Port ( 
           PIX_DATA_IN  : in  std_logic_vector(31 downto 0);
           PIX_DATA_REQ : in  std_logic;
           PIX_DATA_ACK : out std_logic;
           PIX_DATA_SYNC: in  std_logic;
           SEQ_DATA_IN  : in  std_logic_vector(31 downto 0);
           SEQ_DATA_REQ : in  std_logic;
--           SEQ_DATA_ACK : out std_logic;
           SEQ_DATA_SYNC: in  std_logic;
           LOC_DATA_IN  : in  std_logic_vector(15 downto 0);
           LOC_DATA_REQ : in  std_logic;
           LOC_DATA_ACK : out std_logic;
           LOC_DATA_SYNC: in  std_logic;
           DATA_OUT     : out std_logic_vector(31 downto 0);
           DATA_WR      : out std_logic;
           DATA_SYNC    : out std_logic;
           DATA_BUSY    : in  std_logic
         );
end PixDataMuxV40;

architecture Behavioral of PixDataMuxV40 is

begin
------------------------------------------------------------------------
-- Priority encoding for acknowledge strobes
------------------------------------------------------------------------
    PIX_DATA_ACK <= PIX_DATA_REQ and not DATA_BUSY;
--    SEQ_DATA_ACK <= SEQ_DATA_REQ and not (PIX_DATA_REQ or DATA_BUSY);
    LOC_DATA_ACK <= LOC_DATA_REQ and not (PIX_DATA_REQ or SEQ_DATA_REQ or DATA_BUSY);
 
------------------------------------------------------------------------
-- Data mux with priority
------------------------------------------------------------------------
    DATA_OUT <= PIX_DATA_IN when (PIX_DATA_REQ = '1') else                
                SEQ_DATA_IN when (SEQ_DATA_REQ = '1') else
                x"0000" & LOC_DATA_IN;

------------------------------------------------------------------------
-- Sync signals
------------------------------------------------------------------------
    DATA_SYNC <= PIX_DATA_SYNC when (PIX_DATA_REQ = '1') else
                 SEQ_DATA_SYNC when (SEQ_DATA_REQ = '1') else
                 LOC_DATA_SYNC;

------------------------------------------------------------------------
-- Write strobe generation for the fpdp_rm
------------------------------------------------------------------------
    DATA_WR <= not DATA_BUSY and (PIX_DATA_REQ or SEQ_DATA_REQ or LOC_DATA_REQ);

end Behavioral;
