-------------------------------------------------------------------------------
-- Title      : S-Link interface
-- Project    : 
-------------------------------------------------------------------------------
-- File       : SLinkIfc.vhd
-- Author     : Miquel Barcel
-- Company    : IFAE
-- Created    : 200?-??-??
-- Last update: 2011-06-22
-- Platform   : Xilinx
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: Provides the interface between the pixel FPGA and the S-LINK
-- uplink and return link port
-------------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB PIXEL FPGA U25
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   S-LINK INTERFACE
-- VERSION      :   1.0 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK      : SYSTEM CLOCK
-- RESET_IN    : ASYNCHRONOUS MODULE RESET
-- DATA_IN     : 32 BIT DATA TO BE SENT VIA UPLINK TO PAN COMPUTER
-- DATA_WRT    : STROBE TO WRITE DATA_IN INTO MODULE ON RISING EDGE OF SYSCLK
-- DATATYPE    : STROBE SENSED DURING DATA_WRT TO IDENTIFY DATA TYPE - LOW =
-- PIXEL DATA, HI = MESSAGE DATA
-- DATA_BUSY   : SIGNAL TO INHIBIT DATA_WRT STROBE. LINK DOWN OR OVER CAPACITY
-- DATA_OUT    : 32 BIT DATA FROM THE PAN COMPUTER VIA THE RETURN LINK
-- DATA_RDY    : STROBE TO INDICATE DATA AVAILABLE ON DATA_OUT ON RISING EDGE
-- OF SYSCLK
-- DATA_READ   : STROBE TO INDICATE THAT DATA_OUT DATA HAS BEEN LATCHED
-- EXTERNALLY 
-- RESET_OUT   : LATCHED SIGNAL TO IDENTIFY A RESET CONDITION SENT BY THE PAN
-- COMPUTER. RESET BY RESET_IN
--
-- UCLK        : HOLA LSC MASTER CLOCK
-- UD          : HOLA LSC UPLINK DATA PORT
-- UWEN_N      : HOLA LSC WRITE STROBE FOR UD DATA
-- UCTRL_N     : HOLA LSC CONTROL WORD STROBE
-- LRL         : HOLA LSC RETURN DATA PORT
-- LFF_N       : HOLA LSC LINK FULL FLAG
-- UTEST_N     : HOLA LSC LINK LOOP BACK SIGNAL
-- LDOWN_N     : HOLA LSC LINK STATUS SIGNAL
------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 200?-??-??  1.0      MBG     Created
-- 4 Dec 2008           JTO     Replace UCLK output with ODDR flip flop
-- 9 Jun 2009           OBB     Aesthethic changes (signal names,comments,..)
-- 20 Oct 2009          OBB     MCB design contains a DCM which doesn't ouput a
-- valid clock while a reset is performed. Changed all synchrounous reset
-- registers to asynchronous

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library unisim;
use UNISIM.vcomponents.all;

entity SLinkIfc is
  port (
    -- internal fpga input signals
    SYSCLK           : in  std_logic;
    RESET_IN         : in  std_logic;
    DATA_IN          : in  std_logic_vector (31 downto 0);
    DATA_READ        : in  std_logic;
    DATA_WRT         : in  std_logic;
    DATATYPE         : in  std_logic;
    -- input signals from slink connector (fpga input pins)
    LDOWN_N          : in  std_logic;
    LFF_N            : in  std_logic;
    LRL              : in  std_logic_vector (3 downto 0);
    -- Statistics
    clearStats       : in  std_logic;
    statsRcvdDatWrd  : out std_logic_vector(31 downto 0);  -- Rcvd KibiWords
    statsRcvdCtlWrd  : out std_logic_vector(31 downto 0);
    statsXferDatWrd  : out std_logic_vector(31 downto 0);  -- Sended kibiWords
    statsXferCtlWrd  : out std_logic_vector(31 downto 0);
    statsXferDatPag  : out std_logic_vector(31 downto 0);  -- pages (complete
                                                           -- and incomplete)
    statsXferCtlPag  : out std_logic_vector(31 downto 0);
    statsXferDatCPag : out std_logic_vector(31 downto 0);  -- complete pages
    statsXferCtlCPag : out std_logic_vector(31 downto 0);
    statsLFFErr      : out std_logic_vector(15 downto 0);
    statsLDownErr    : out std_logic_vector(15 downto 0);
    statsUpTimeMin   : out std_logic_vector(15 downto 0);
    statsUpTimeSec   : out std_logic_vector(5 downto 0);
    infoPageLength   : out std_logic_vector(15 downto 0);
    infoMaxWait      : out std_logic_vector(15 downto 0);

    -- output signals to internal fpga design
    DATA_BUSY : out std_logic;          -- fifoXferFull
    DATA_OUT  : out std_logic_vector (31 downto 0);
    DATA_RDY  : out std_logic;
    RESET_OUT : out std_logic;
    -- output signals to slink connector (fpga output pins)
    UCLK      : out std_logic;
    UCTRL_N   : out std_logic;
    UD        : out std_logic_vector (31 downto 0);
    UTEST_N   : out std_logic;
    URESET_N  : out std_logic;
    UWEN_N    : out std_logic;
    -- led outputs
    activityLed : out std_logic;
    linkUpLed   : out std_logic;
    xoffLed     : out std_logic);
end SLinkIfc;

architecture BEHAVIORAL of SLinkIfc is
  component lrl_decoder
    port (
      areset    : in  std_logic;
      sysclk    : in  std_logic;
      lrl       : in  std_logic_vector (3 downto 0);
      reset_out : out std_logic;
      data      : out std_logic_vector (31 downto 0);
      wrden     : out std_logic;
      wrcen     : out std_logic
      );
  end component;

  component slink_fifo
    generic (
      width : integer := 32;
      depth : integer := 6
      );
    port (
      clk    : in  std_logic;
      areset : in  std_logic;
      din    : in  std_logic_vector (width-1 downto 0);
      wr     : in  std_logic;
      dout   : out std_logic_vector (width-1 downto 0);
      rd     : in  std_logic;
      empty  : out std_logic;
      full   : out std_logic
      );
  end component;




  signal lrlRcvdData     : std_logic_vector(31 downto 0);
  signal lrlWrData       : std_logic;
  signal lrlWrCtrl       : std_logic;
  signal fifoRcvdEmpty   : std_logic;
  signal fifoRcvdDataOut : std_logic_vector(31 downto 0);
  signal fifoRcvdDataRd  : std_logic;

  signal data2send     : std_logic_vector(31 downto 0);
  signal data2sendType : std_logic;

  signal xferDataWr      : std_logic;
  signal fifoXferFull    : std_logic;
  signal fifoXferDataOut : std_logic_vector(31 downto 0);
  signal fifoXferEmpty   : std_logic;
  signal fifoXferRd      : std_logic;
  signal xferDataType    : std_logic;

  signal wordsInCurrPage : std_logic_vector(15 downto 0);
  signal pageLength      : std_logic_vector(15 downto 0);
  signal pageLengthWr    : std_logic;
  signal pageFull        : std_logic;
  signal wordCountEna    : std_logic;
  signal wordsInCurrPageRst : std_logic;

  signal maxWait      : std_logic_vector(15 downto 0);
  signal waitCounter  : std_logic_vector(15 downto 0);
  signal waitFlag     : std_logic;
  signal waitCountEna : std_logic;
  signal waitCountRst : std_logic;
  signal maxWaitWr    : std_logic;

  signal loopMode         : std_logic;
  signal loopModeDataType : std_logic;
  signal loopModeWr       : std_logic;

  signal rLFF_N, rrLFF_N, rLDOWN_N, rrLDOWN_N : std_logic;

  signal linkdown : std_logic := '0';   -- positive logic
  signal linkdownreset_n : std_logic;   -- reset generated due to ldown_n

  signal tmpUWEN_n, tmpUCtrl_n : std_logic;
  signal tempUD                             : std_logic_vector(31 downto 0);

  signal muxUd : std_logic_vector(31 downto 0);

  signal areset_n : std_logic;
  signal mixreset_n : std_logic;        -- reset coming from out of module or
                                        -- due to linkdown
  type sm_type is (
    reset,
    idle,
    ldown,
    ctrl_scw,
    ctrl_send,
    ctrl_ecw,
    data_scw,
    data_send,
    data_ecw);
  signal state, next_state : sm_type;

  -- statistics registers
  signal iStatsRcvdDatWrd  : unsigned(31 downto 0);
  signal iStatsRcvdCtlWrd  : unsigned(31 downto 0);
  signal iStatsXferDatWrd  : unsigned(31 downto 0);
  signal iStatsXferCtlWrd  : unsigned(31 downto 0);
  signal iStatsXferDatPag  : unsigned(31 downto 0);  -- pages (complete
                                                     -- and incomplete)
  signal iStatsXferCtlPag  : unsigned(31 downto 0);
  signal iStatsXferDatCPag : unsigned(31 downto 0);  -- complete pages
  signal iStatsXferCtlCPag : unsigned(31 downto 0);
  signal iStatsLFFErr      : unsigned(15 downto 0);
  signal iStatsLDownErr    : unsigned(15 downto 0);
  signal iStatsUpTimeMin   : unsigned(15 downto 0);
  signal iStatsUpTimeSec   : unsigned(5 downto 0);
--  signal infoPageLength : std_logic_vector(15 downto 0);
--  signal infoMaxWait : std_logic_vector(15 downto 0);
  
  
  


begin

  
  
  -----------------------------------------------------------------------------
  -- LEDS
  -----------------------------------------------------------------------------
  -- activity Led
  activityLed <= lrlWrCtrl or lrlWrData or fifoXferRd;
  -- link up led
  linkUpLed <= LDOWN_N;
  -- xoff led
  xoffLed <= not LFF_N;

  -----------------------------------------------------------------------------
  -- LRL decoder instance
  -----------------------------------------------------------------------------
  -- Component which decodes lrl lines received data and stores it to
  -- fifoRcvData if it is a data word or sets loopMode, pageLength or
  -- reg_numwait if it is a control word.
  decoder : lrl_decoder
    port map (
      areset    => mixreset_n,
      sysclk    => SYSCLK,
      lrl       => LRL,
      data      => lrlRcvdData,
      wrden     => lrlWrData,
      wrcen     => lrlWrCtrl,
      reset_out => RESET_OUT
      );


  -- Fifo to store data received through lrl lines before is read from other
  -- component of the design
  fifoRcvData : slink_fifo
    generic map (
      width => 32,
      depth => 6)
    port map (
      areset => mixreset_n,
      clk    => SYSCLK,
      din    => lrlRcvdData,
      wr     => lrlWrData,
      dout   => fifoRcvdDataOut,
      rd     => fifoRcvdDataRd,
      empty  => fifoRcvdEmpty
      );

  -- Fifo for data to send with Slink
  fifoXferData : slink_fifo
    generic map (
      width => 32,
      depth => 6)
    port map (
      areset => mixreset_n,
      clk    => SYSCLK,
      din    => data2send,
      wr     => xferDataWr,
      dout   => fifoXferDataOut,
      full   => fifoXferFull,
      rd     => fifoXferRd,
      empty  => fifoXferEmpty
      );

  -- Fifo to store which kind of transfer it is (control or data word)
  fifoXferType : slink_fifo
    generic map (
      width => 1,
      depth => 6)
    port map (
      areset  => mixreset_n,
      clk     => SYSCLK,
      din(0)  => data2sendType,
      wr      => xferDataWr,
      dout(0) => xferDataType,
      rd      => fifoXferRd
      );

  -- Loop mode: Return FIFO
  DATA_OUT <= X"1BACAFEA" when loopMode = '1' else
              fifoRcvdDataOut;
  DATA_RDY <= '0' when loopMode = '1' else
              not fifoRcvdEmpty;
  fifoRcvdDataRd <= not fifoXferFull when loopMode = '1' else
                    DATA_READ;
  -- Loop mode: Forward FIFO
  DATA_BUSY <= '1' when loopMode = '1' else
               fifoXferFull;
  xferDataWr <= not fifoRcvdEmpty when loopMode = '1' else
                DATA_WRT;
  data2send <= fifoRcvdDataOut when loopMode = '1' else
               DATA_IN;
  data2sendType <= loopModeDataType when loopMode = '1' else
                   DATATYPE;


  reg_inputs : process (sysclk, areset_n)
  begin  -- process reg_inputs
    if areset_n = '0' then                -- asynchronous reset (active low)
      rLFF_N    <= '0';
      rrLFF_N   <= '0';
      rLDOWN_N  <= '0';
      rrLDOWN_N <= '0';
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      rLFF_N    <= LFF_N;
      rrLFF_N   <= rLFF_N;
      rLDOWN_N  <= LDOWN_N;
      rrLDOWN_N <= rLDOWN_N;
    end if;
  end process reg_inputs;

  register_num_page : process (sysclk, areset_n)
  begin  -- process register_num_page
    if areset_n = '0' then                -- asynchronous reset (active low)
      pageLength <= x"0FFE";
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if pageLengthWr = '1' then
        pageLength <= lrlRcvdData(15 downto 0);
      end if;
    end if;
  end process register_num_page;

  register_num_wait : process (sysclk, areset_n)
  begin  -- process register_num_wait
    if areset_n = '0' then                -- asynchronous reset (active low)
      maxWait <= x"3FFF";
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if maxWaitWr = '1' then
        maxWait <= lrlRcvdData(15 downto 0);
      end if;
    end if;
  end process register_num_wait;

  register_loopMode : process (sysclk, areset_n)
  begin  -- process register_loopMode
    if areset_n = '0' then                -- asynchronous reset (active low)
      loopMode         <= '0';
      loopModeDataType <= '0';
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if loopModeWr = '1' then
        loopMode         <= lrlRcvdData(0);
        loopModeDataType <= lrlRcvdData(1);
      end if;
    end if;
  end process register_loopMode;


  counter_page : process (sysclk, areset_n)
  begin  -- process counter_page
    if areset_n = '0' then                -- asynchronous reset (active low)
      wordsInCurrPage <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if wordsInCurrPageRst = '1' then
        wordsInCurrPage <= (others => '0');
      elsif wordCountEna = '1' then
        wordsInCurrPage <= std_logic_vector(unsigned(wordsInCurrPage) + 1);
      end if;
    end if;
  end process counter_page;

  counter_wait : process (sysclk, areset_n)
  begin  -- process counter_wait
    if areset_n = '0' then                -- asynchronous reset (active low)
      waitCounter <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if waitCountRst = '1' then
        waitCounter <= (others => '0');
      elsif waitCountEna = '1' then
        waitCounter <= std_logic_vector(unsigned(waitCounter) + 1);
      end if;
    end if;
  end process counter_wait;

  ---------------------------------------------------------------------------
  --  State Machine
  ---------------------------------------------------------------------------

  next_state <=
    reset     when areset_n = '0' else
    reset     when state = reset and rrLDOWN_N = '0' else
    idle      when state = reset else
    -- if link is down but not on reset state, go to ldown state
    ldown     when linkdown = '1' else
    idle      when state = ldown else
    -- if rrLFF_N = '0' it remains on the same state
    state     when rrLFF_N = '0' else
    idle      when state = idle and fifoXferEmpty = '1' else
    ctrl_scw  when state = idle and xferDataType = '1' else
    data_scw  when state = idle and xferDataType = '0' else
    ctrl_send when state = ctrl_scw else
    ctrl_ecw  when state = ctrl_send and fifoXferEmpty = '1' else
    ctrl_ecw  when state = ctrl_send and xferDataType = '0' else
    ctrl_send when state = ctrl_send else
--    ctrl_ecw  when state = ctrl_send else
    data_send when state = data_scw else
    data_ecw  when state = data_send and xferDataType = '1' else
    data_ecw  when state = data_send and waitFlag = '1'else
    data_ecw when (state = data_send and
                   pageFull = '1' and
                   fifoXferEmpty = '0') else
    data_send when state = data_send else
    idle;

  state_machine : process (sysclk, areset_n)
  begin  -- process state_machine
    if sysclk'event and sysclk = '1' then  -- rising clock edge
      if areset_n = '0' then
        state <= reset;
      else
        state <= next_state;
      end if;
    end if;
  end process state_machine;

  fifoXferRd <=
    '0' when rrLFF_N = '0' else
    '0' when fifoXferEmpty = '1' else
    '1' when state = ctrl_send and xferDataType = '1' else
    -- should be "when state = ctrl_send and next_state = ctrl_send"
    -- No! Look to it deeply
    '1' when state = data_send and xferDataType = '0' and pageFull = '0' else
    -- should be "when state = data_send and next_state = data_send"
    -- Noooo! It can be on state = data_send and not sending words because it
    -- is waiting for more words. We read from the fifo only when we have to
    -- send data (and have data to send!!)
    '0';

  loopModeWr <=
    '1' when lrlWrCtrl = '1' and lrlRcvdData(31 downto 28) = "0010" else
    '0';

  pageLengthWr <=
    '1' when lrlWrCtrl = '1' and lrlRcvdData(31 downto 28) = "0001" else
    '0';

  maxWaitWr <=
    '1' when lrlWrCtrl = '1' and lrlRcvdData(31 downto 28) = "0000" else
    '0';

  waitFlag <=
    '1' when waitCounter = maxWait else
    '0';

  pageFull <=
    '1' when wordsInCurrPage = pageLength else
    '0';

  wordsInCurrPageRst <=
    '0' when state = data_send else
    '1';

  waitCountRst <=
    '0' when state = data_send and fifoXferEmpty = '1' else
    '1';

  wordCountEna <=
    '1' when rrLFF_N = '1' and fifoXferEmpty = '0' and state = data_send else
    '0';

  waitCountEna <=
    '1' when rrLFF_N = '1' and fifoXferEmpty = '1' and state = data_send else
    '0';

  -----------------------------------------------------------------------------
  -- linkdown and linkdownreset_n generation
  -----------------------------------------------------------------------------
  linkdown <= not rLDOWN_N;
    --'1' when (rrLDOWN_N and rLDOWN_N) = '0' else
    --'0';
  --linkdownreset_n
  ldownresetproc: process (sysClk, areset_n)
    variable tickCounts : integer range 0 to (2**15-1) := 0;
  begin  -- process ldownresetproc
    if areset_n = '0' then              -- asynchronous reset (active low)
      linkdownreset_n <= '0';
      tickCounts := 0;
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if state = ldown then
        -- update counter
        if tickCounts = 20000000 then
          tickCounts := 0;
        else
          tickCounts := tickCounts + 1;
        end if;
        -- update linkdownreset_n signal
        if tickCounts < 100 then
          linkdownreset_n <= '0';
        else
          linkdownreset_n <= '1';
        end if;
      else
        linkdownreset_n <= '1';
        tickCounts := 0;
      end if;
    end if;
  end process ldownresetproc;

  mixreset_n <= areset_n and linkdownreset_n;
  
  -----------------------------------------------------------------------------
  -- Output registers
  -----------------------------------------------------------------------------
  with state select
    muxUd <=
    X"B0F0C000"     when ctrl_scw,
    fifoXferDataOut when ctrl_send,
    X"E0F0C000"     when ctrl_ecw,
    X"B0F0D000"     when data_scw,
    fifoXferDataOut when data_send,
    X"E0F0D000"     when data_ecw,
    X"0BADDA7A"     when others;

  tempUD <= X"0BADDA7A" when rrLFF_N = '0' else
            muxUd;

  tmpUCtrl_n <=
    '0' when state = ctrl_scw else
    '0' when state = ctrl_ecw else
    '0' when state = data_scw else
    '0' when state = data_ecw else
    '1';
  tmpUWEN_n <=
    '1' when rrLFF_N = '0' else
    '0' when state = ctrl_scw else
    '0' when state = ctrl_send and fifoXferRd = '1' else
    '0' when state = ctrl_ecw else
    '0' when state = data_scw else
    '0' when state = data_send and fifoXferRd = '1' else
    '0' when state = data_ecw else
    '1';
  UReset_n <=
    '0' when state = reset else
    linkdownreset_n when state = ldown else
    '1';

  registers_output : process (SYSCLK, areset_n)
  begin  -- process registers_output
    if SYSCLK'event and SYSCLK = '1' then
      if areset_n = '0' then
        UD       <= (others => '0');
        UCTRL_N  <= '1';
        UWEN_N   <= '1';
      else
        UD       <= tempUD;
        UCTRL_N  <= tmpUCtrl_n;
        UWEN_N   <= tmpUWEN_n;
      end if;
    end if;
  end process registers_output;

  areset_n  <= not RESET_IN;
  UTEST_N <= '1';

  -- use a DDR output register, which works like this

  -- OPPOSITE_EDGE mode - The data transmit interface uses the classic DDR
  -- methodology. 
  -- Given a data and clock at pin D1-2 and C respectively, D1 is sampled at
  -- every positive edge of clock C, and D2 is sampled at every negative edge
  -- of clock C. 
  -- Q changes every clock edge.

  ODDR_inst : ODDR
    generic map(
      DDR_CLK_EDGE => "OPPOSITE_EDGE",  -- "OPPOSITE_EDGE" or "SAME_EDGE"
      INIT         => '0',  -- Initial value for Q port (1 or 0)
      SRTYPE       => "SYNC"            -- Reset Type ("ASYNC" or "SYNC")
      )
    port map(
      C  => sysclk,
      CE => '1',
      D1 => '0',
      D2 => '1',
      R  => '0',
      S  => '0',
      Q  => UCLK
      );




  -----------------------------------------------------------------------------
  -- Statistics registers
  -----------------------------------------------------------------------------
--  signal iStatsRcvdDatWrd : std_logic_vector(31 downto 0);
--  signal iStatsRcvdCtlWrd : std_logic_vector(31 downto 0);
  iStatsRcvdWords : process (sysclk, areset_n)
  begin  -- process iStatsRcvdDataWords
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsRcvdDatWrd <= (others => '0');
      iStatsRcvdCtlWrd <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsRcvdDatWrd <= (others => '0');
        iStatsRcvdCtlWrd <= (others => '0');
      else
        if lrlWrData = '1' then
          iStatsRcvdDatWrd <= iStatsRcvdDatWrd + 1;
        end if;
        if lrlWrCtrl = '1' then
          iStatsRcvdCtlWrd <= iStatsRcvdCtlWrd + 1;
        end if;
      end if;
    end if;
  end process iStatsRcvdWords;
  -- output values
  statsRcvdDatWrd <= std_logic_vector(iStatsRcvdDatWrd(31 downto 0));
  statsRcvdCtlWrd <= std_logic_vector(iStatsRcvdCtlWrd(31 downto 0));

--  signal iStatsXferDatWrd : std_logic_vector(41 downto 0);
--  signal iStatsXferCtlWrd : std_logic_vector(41 downto 0);
  iStatsXferWords : process (sysclk, areset_n)
  begin  -- process iStatsXferWords
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsXferDatWrd <= (others => '0');
      iStatsXferCtlWrd <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsXferDatWrd <= (others => '0');
        iStatsXferCtlWrd <= (others => '0');
      elsif xferDataWr = '1' then
        if data2sendType = '1' then
          iStatsXferCtlWrd <= iStatsXferCtlWrd + 1;
        else
          iStatsXferDatWrd <= iStatsXferDatWrd + 1;
        end if;
      end if;
    end if;
  end process iStatsXferWords;
  -- outputs
  statsXferDatWrd <= std_logic_vector(iStatsXferDatWrd(31 downto 0));
  statsXferCtlWrd <= std_logic_vector(iStatsXferCtlWrd(31 downto 0));

--  signal iStatsXferDatPag : std_logic_vector(31 downto 0);  -- pages (complete
--                                                           -- and incomplete)
--  signal iStatsXferCtlPag : std_logic_vector(31 downto 0);
  iStatsXferPages : process (sysclk, areset_n)
  begin  -- process iStatsXferPages
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsXferCtlPag <= (others => '0');
      iStatsXferDatPag <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsXferDatPag <= (others => '0');
        iStatsXferCtlPag <= (others => '0');
      else
        if state = ctrl_send and next_state = ctrl_ecw then
          iStatsXferCtlPag <= iStatsXferCtlPag + 1;
        end if;
        if state = data_send and next_state = data_ecw then
          iStatsXferDatPag <= iStatsXferDatPag + 1;
        end if;
      end if;
    end if;
  end process iStatsXferPages;
  -- output
  statsXferDatPag <= std_logic_vector(iStatsXferDatPag);
  statsXferCtlPag <= std_logic_vector(iStatsXferCtlPag);

--  signal iStatsXferDatCPag : std_logic_vector(31 downto 0);  -- complete pages
--  signal iStatsXferCtlCPag : std_logic_vector(31 downto 0);
  iStatsXferCompletePages : process (sysclk, areset_n)
  begin  -- process iStatsXferPages
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsXferCtlCPag <= (others => '0');
      iStatsXferDatCPag <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsXferDatCPag <= (others => '0');
        iStatsXferCtlCPag <= (others => '0');
      else
        if state = ctrl_send and pageFull = '1' and fifoXferEmpty = '0' then
          iStatsXferCtlCPag <= iStatsXferCtlCPag + 1;
        end if;
        if state = data_send and pageFull = '1' and fifoXferEmpty = '0' then
          iStatsXferDatCPag <= iStatsXferDatCPag + 1;
        end if;
      end if;
    end if;
  end process iStatsXferCompletePages;
  -- outputs
  statsXferDatCPag <= std_logic_vector(iStatsXferDatCPag);
  statsXferCtlCPag <= std_logic_vector(iStatsXferCtlCPag);

--  signal iStatsLFFErr : std_logic_vector(15 downto 0);
  iStatsLinkFullFlag : process (sysclk, areset_n)
  begin  -- process iStatsLinkFullFlag
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsLFFErr <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsLFFErr <= (others => '0');
      elsif rrLFF_N = '1' and rLFF_N = '0' then
        -- LFF_N goes down
        iStatsLFFErr <= iStatsLFFErr + 1;
      end if;
    end if;
  end process iStatsLinkFullFlag;
  -- output
  statsLFFErr <= std_logic_vector(iStatsLFFErr);

--  signal iStatsLDownErr : std_logic_vector(15 downto 0);
  iStatsLinkDown : process (sysclk, areset_n)
  begin  -- process iStatsLinkDown
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsLDownErr <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clearStats = '1' then
        iStatsLDownErr <= (others => '0');
      elsif rrLDOWN_N = '1' and rLDOWN_N = '0' then
        -- LDOWN_N goes down
        iStatsLDownErr <= iStatsLDownErr + 1;
      end if;
    end if;
  end process iStatsLinkDown;
  --output
  statsLDownErr <= std_logic_vector(iStatsLDownErr);

--  signal iStatsUpTimeMin : std_logic_vector(15 downto 0);
--  signal iStatsUpTimeSec : std_logic_vector(5 downto 0);
  iStatsUpTime : process (sysclk, areset_n)
    variable tickCounts : integer range 0 to 67108863 := 0;
  begin  -- process iStatsUpTime
    if areset_n = '0' then                -- asynchronous reset (active low)
      iStatsUpTimeMin <= (others => '0');
      iStatsUpTimeSec <= (others => '0');
      tickCounts      := 0;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if tickCounts = 40000000 then     -- sysclk at 40MHz
        tickCounts := 0;
        if iStatsUpTimeSec = 59 then
          iStatsUpTimeSec <= (others => '0');
          iStatsUpTimeMin <= iStatsUpTimeMin + 1;
        else
          iStatsUpTimeSec <= iStatsUpTimeSec + 1;
        end if;
      else
        tickCounts := tickCounts + 1;
      end if;
    end if;
  end process iStatsUpTime;
  -- output
  statsUpTimeSec <= std_logic_vector(iStatsUpTimeSec);
  statsUpTimeMin <= std_logic_vector(iStatsUpTimeMin);

--  signal infoPageLength : std_logic_vector(15 downto 0);
  infoPageLength <= pageLength;
--  signal infoMaxWait : std_logic_vector(15 downto 0);
  infoMaxWait    <= maxWait;
end BEHAVIORAL;
