-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- ????-??-??           MBG     Created
-- 2009-11-20           OBB     Changed all registers to asynchronous reset
library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity LRL_DECODER is
-- clock cycles to wait after a change in the LRL to make sure they are stable
  port (
    areset : in std_logic;
    sysclk : in std_logic;

-- Slink inputs
    lrl : in std_logic_vector (3 downto 0);

-- Data and control bits outputs
    data      : out std_logic_vector (31 downto 0);
    wrden     : out std_logic;
    wrcen     : out std_logic;
    reset_out : out std_logic
    );
end LRL_DECODER;

architecture arch of LRL_DECODER is
  signal in_lrl, reg_lrl, rreg_lrl : std_logic_vector(3 downto 0);
  signal lrl_rise, lrl_fall        : std_logic_vector(3 downto 0);
  signal clear_edges               : std_logic;
  signal shiftreg                  : std_logic_vector(35 downto 0);
  signal shiftreg_ena              : std_logic;
  signal cnt_clk                   : std_logic_vector(7 downto 0);
  signal cnt_clk_ena, cnt_clk_clr  : std_logic;
  signal reg_nclk                  : std_logic_vector(7 downto 0);
  signal reg_nclk_load             : std_logic;
  signal wrcen_int, reset_int      : std_logic;

  type   sm_type is (idle, wait_nclk, proc);
  signal state, next_state : sm_type;

begin


  lrl_register : process (areset, lrl, sysclk)
  begin  -- process lrl_register
    if areset = '0' then                -- asynchronous reset (active low)
      in_lrl   <= lrl;
      reg_lrl  <= lrl;
      rreg_lrl <= lrl;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      in_lrl   <= lrl;
      reg_lrl  <= in_lrl;
      rreg_lrl <= reg_lrl;
    end if;
  end process lrl_register;

  lrl_edgedetect : process (sysclk, areset)
  begin  -- process lrl_edgedetect
    if areset = '0' then                -- asynchronous reset (active low)
      lrl_fall <= (others => '0');
      lrl_rise <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if clear_edges = '1' then
        lrl_rise <= (others => '0');
        lrl_fall <= (others => '0');
      else
        lrl_rise <= lrl_rise or (reg_lrl and not rreg_lrl);
        lrl_fall <= lrl_fall or (not reg_lrl and rreg_lrl);
      end if;
    end if;
  end process lrl_edgedetect;

  shift_register : process (sysclk, areset)
  begin  -- process shift_register
    if areset = '0' then                -- asynchronous reset (active low)
      shiftreg <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if shiftreg_ena = '1' then
        shiftreg(35 downto 0) <= reg_lrl(2 downto 0) &
                                 shiftreg(35 downto 3);
      end if;
    end if;
  end process shift_register;

  counter_clk : process (sysclk, areset)
  begin  -- process counter_clk
    if areset = '0' then                -- asynchronous reset (active low)
      cnt_clk <= (others => '0');
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if cnt_clk_clr = '1' then
        cnt_clk <= (others => '0');
      elsif cnt_clk_ena = '1' then
        cnt_clk <= std_logic_vector(unsigned(cnt_clk)+1);
      end if;
    end if;
  end process counter_clk;

  register_nclk : process (sysclk, areset)
  begin  -- process register_nclk
    if areset = '0' then                -- asynchronous reset (active low)
      reg_nclk <= x"04";
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      if reg_nclk_load = '1' then
        reg_nclk <= shiftreg(7 downto 0);
      end if;
    end if;
  end process register_nclk;

  -----------------------------------------------------------------------------
  -- State machine
  -----------------------------------------------------------------------------

  next_state <= idle when areset = '0' else
                wait_nclk when state = idle and (lrl_rise /= X"0" or lrl_fall /= X"0") else
                proc      when state = wait_nclk and cnt_clk = reg_nclk                else
                wait_nclk when state = wait_nclk                                       else
                idle;

  state_machine : process (sysclk, areset)
  begin  -- process state_machine
    if areset = '0' then                -- asynchronous reset (active low)
      state <= idle;
    elsif sysclk'event and sysclk = '1' then  -- rising clock edge
      state <= next_state;
    end if;
  end process state_machine;

  shiftreg_ena <= '1' when state = proc and (lrl_rise(3) = '1'
                                             or lrl_fall (3) = '1')
                  else '0';
  
  cnt_clk_ena <= '1' when state = wait_nclk
                 else '0';

  cnt_clk_clr <= '1' when state = wait_nclk and cnt_clk = reg_nclk
                 else '0';

  wrden <= '1' when state = proc and (lrl_rise(3) = '0'
                                      and lrl_fall(3) = '0'
                                      and lrl_rise (2) = '1')
           else '0';

  wrcen_int <= '1' when state = proc and (lrl_rise(3) = '0'
                                          and lrl_fall(3) = '0'
                                          and lrl_rise (1) = '1')
               else '0';

  reg_nclk_load <= '1' when wrcen_int = '1' and shiftreg(31) = '1'
                   else '0';

  wrcen <= '1' when wrcen_int = '1' and shiftreg(31) = '0'
           else '0';

  reset_int <= '1' when state = proc and (lrl_rise(3) = '0'
                                          and lrl_fall(3) = '0'
                                          and lrl_fall (0) = '1')
               else '0';

  clear_edges <= '1' when state = proc
                 else '0';


  -----------------------------------------------------------------------------
  -- OBB: Output reset_int
  reset_out <= reset_int;
  -----------------------------------------------------------------------------

  data (31 downto 0) <= shiftreg (31 downto 0);



end arch;
