------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   CLOCK ENABLE REGISTER
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   CLOCKENABLEREGISTER.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/06/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :  PROVIDES CLOCK DRIVERS FOR THE INDIVIDUAL BOARD CLOCKS. 
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- Version 3.4 .. Revision 0 hardware and 8 slot schroff backplane only.
-- CURRENT 8 SLOT BACKPLANE HAS BOARD CLOCKS 2 AND 3, 4 AND 5, BUSSED TOGETHER
-- THIS IMPLEMENTATION HAS SPECIFIC CODING FOR THIS SCHEME. SETTING BIT 1 ON
-- WILL RESULT IN BOTH SLOT 2 AND 3 HAVING CLOCKS.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 02/26/2003 PETER MOORE  0.1  ORIGINAL VERSION
-- 03/26/2004 PETER MOORE  3.4  CHANGED TO NON-TRISTATE DRIVERS AND INVERTED CLOCK
--                              TO COMPENSATE FOR PROPOGATION DELAY ON PCI BACKPLANE
-- 05/06/2004 PETER MOORE  4.0  ASSIGNMENTS CHANGED FOR 6 SLOT BACKPLANE WITH REV A HARDWARE.
-- 4 Dec 08   JTO               don't gate output clocks!  use DDR output registers
-- 
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK         : SYSTEM CLOCK
-- RST         : SYSTEM RESET
-- WRT         : WRITE STROBE FOR CLK_EN_DATA
-- CLK_EN_DATA : LOWEST 7 BITS CONTROL THE ENABLES
-- CLK_DATA_OUT: REFLECTION OF CLOCK REGISTER CONTENTS
-- CLK2        : BOARD 2 CLOCK DRIVER OUTPUT
-- CLK3        : BOARD 3 CLOCK DRIVER OUTPUT
-- CLK4        : BOARD 4 CLOCK DRIVER OUTPUT
-- CLK5        : BOARD 5 CLOCK DRIVER OUTPUT
-- CLK6        : BOARD 6 CLOCK DRIVER OUTPUT
-- CLK7        : BOARD 7 CLOCK DRIVER OUTPUT
-- CLK8        : BOARD 8 CLOCK DRIVER OUTPUT
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

library unisim;
use UNISIM.vcomponents.all;

entity ClockEnableRegister is
  port (CLK           : in  std_logic;
         RST          : in  std_logic;
         WRT          : in  std_logic;
         CLK_EN_DATA  : in  std_logic_vector(15 downto 0);
         CLK_DATA_OUT : out std_logic_vector(15 downto 0);
         bp_clk       : out std_logic_vector(8 downto 2)
--         CLK2         : out std_logic;
--         CLK3         : out std_logic;
--         CLK4         : out std_logic;
--         CLK5         : out std_logic;
--         CLK6         : out std_logic;
--         CLK7         : out std_logic;
--         CLK8         : out std_logic
         );
end ClockEnableRegister;

architecture Behavioral of ClockEnableRegister is

  signal clk_reg  : std_logic_vector(15 downto 0);
  -- signal clk_n    : std_logic;
  signal en_clk   : std_logic_vector(8 downto 2);
  signal oddr_out : std_logic_vector(8 downto 2);

begin

------------------------------------------------------------------------
-- Update the clock enable register through reset or write strobe
------------------------------------------------------------------------
--  clk_wr : process (CLK)
--  begin
--    if (rising_edge(CLK)) then
--      if (RST = '1') then
--        clk_reg <= (others => '0');
--      elsif (WRT = '1') then
--        clk_reg <= CLK_EN_DATA;
--      end if;
--    end if;
--  end process;

  clk_wr: process (clk, rst)
  begin  -- process clk_wr
    if rst = '1' then                   -- asynchronous reset (active high)
      clk_reg <= (others => '0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if wrt = '1' then
        clk_reg <= CLK_EN_DATA;
      end if;
    end if;
  end process clk_wr;
  
------------------------------------------------------------------------
-- Let the rest of the world see the enables data (as it was written)
------------------------------------------------------------------------
  CLK_DATA_OUT <= clk_reg;

------------------------------------------------------------------------
-- Decode the enables depending on 8 slot or 6 slot backplane
------------------------------------------------------------------------
  en_clk(2) <= clk_reg(1) when (clk_reg(15) = '0') else clk_reg(9);
  en_clk(3) <= clk_reg(2) when (clk_reg(15) = '0') else clk_reg(8);
  en_clk(4) <= clk_reg(3) when (clk_reg(15) = '0') else (clk_reg(1) or clk_reg(2));
  en_clk(5) <= clk_reg(4) when (clk_reg(15) = '0') else clk_reg(5);
  en_clk(6) <= clk_reg(5) when (clk_reg(15) = '0') else clk_reg(6);
  en_clk(7) <= clk_reg(6) when (clk_reg(15) = '0') else (clk_reg(3) or clk_reg(4));
  en_clk(8) <= clk_reg(7) when (clk_reg(15) = '0') else clk_reg(7);

-------------------------------------------------------------------------------
-- Use a DDR output register
-------------------------------------------------------------------------------
  -- DDR output register works like this:
  -- OPPOSITE_EDGE mode - The data transmit interface uses the classic DDR
  -- methodology. 
  -- Given a data and clock at pin D1-2 and C respectively, D1 is sampled at
  -- every positive edge of clock C, and D2 is sampled at every negative edge
  -- of clock C. 
  -- Q changes every clock edge.

  gen_oddr : for i in 2 to 8 generate
    ODDR_inst : ODDR
      generic map(
        DDR_CLK_EDGE => "OPPOSITE_EDGE",  -- "OPPOSITE_EDGE" or "SAME_EDGE"
        INIT         => '0',  -- Initial value for Q port (1 or 0)
        SRTYPE       => "ASYNC"          -- Reset Type ("ASYNC" or "SYNC")
        )
      port map(
        C  => CLK,
        CE => en_clk(i),
        D1 => '1',
        D2 => '0',
        R  => '0',
        S  => '0',
        Q  => oddr_out(i)
        );
  end generate gen_oddr;

  bp_clk(8 downto 2) <= oddr_out(8 downto 2);
  
end Behavioral;
