------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (IR ACQ FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   DECODE EXECUTION UNIT
-- VERSION      :   3.1 
------------------------------------------------------------------------
-- FILE NAME    :   DecodeExecuteV2.vhd
-- AUTHOR       :   PETER MOORE
-- DATE         :   10/28/2005
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- Deocdes and executes the instruction based on the instruction 
-- definition as specified in the Instruction Set doccument. 
-- The Pattern Pointer which outputs the sequence pattern  
-- is controlled by execution of the data transfer instructions 
-- along with the Clock, Mode, Dev Addr and Board Sel registers.   
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- Currently the Acknowledge & Request HandShake is commented.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE       AUTHOR  VERSION  MODIFICATIONS
-- 02/20/2003  KAVI     1.0    ORIGINAL VERSION
-- 05/13/2004  PCM      2.0    ADDED COMMENTS + INITIALIZATION CODE +
--                             PATTERN POINTER PUSH / POP + SELECT AUTO 
--                             CANCEL FEATURES + NEW LPB COMMAND (TAKES OPERAND
--                             FROM PATTERN MEMORY INSTEAD OF PGM MEMORY) + NON 
--                             BLOCKING DELAY COUNTERS.
-- 12/06/2004  PCM      3.0    ADDED LSR REGISTER CLEAR ON SEQUENCER NOT ACTIVE.
-- 10/28/2005  PCM      3.1    ADDED LOCAL SELECT LINE OUT TO PROVIDE FOR EVENT REGISTER
-- 11-Dec-08   JTO             add sync reset for all registers, remove inouts
------------------------------------------------------------------------
-- PORT DEFINITIONS :

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity DecodeExecute is
  port (
    seqclk             : in  std_logic;
    sysclk             : in  std_logic;
    reset              : in  std_logic;
    SeqRunFlag         : in  std_logic;
    CmdReg             : in  std_logic_vector(15 downto 0);
    OPCODEL            : in  std_logic_vector(15 downto 0);
    OPCODEH            : in  std_logic_vector(15 downto 0);
    LoopReg0           : in  std_logic_vector(15 downto 0);
    LoopReg1           : in  std_logic_vector(15 downto 0);
    LoopReg2           : in  std_logic_vector(15 downto 0);
    LoopReg3           : in  std_logic_vector(15 downto 0);
    LoopReg4           : in  std_logic_vector(15 downto 0);
    LoopReg5           : in  std_logic_vector(15 downto 0);
    LoopReg6           : in  std_logic_vector(15 downto 0);
    LoopReg7           : in  std_logic_vector(15 downto 0);
    LoopReg8           : in  std_logic_vector(15 downto 0);
    LoopReg9           : in  std_logic_vector(15 downto 0);
    LoopReg10          : in  std_logic_vector(15 downto 0);
    LoopReg11          : in  std_logic_vector(15 downto 0);
    LoopReg12          : in  std_logic_vector(15 downto 0);
    LoopReg13          : in  std_logic_vector(15 downto 0);
    LoopReg14          : in  std_logic_vector(15 downto 0);
    LoopReg15          : in  std_logic_vector(15 downto 0);
    PatternData        : in  std_logic_vector(15 downto 0);
    SeqEnablesWrite    : out std_logic;  -- WHEN BOARD SELECT = 1, WRITE TO THE ENABLES REGISTER FROM PATTERN MEMORY.
    AutoCancelFlag     : in  std_logic;  -- Used to automatically cancel board selects when set.
    PANADDR            : in  std_logic_vector(15 downto 0);
    ProgPtrA           : out std_logic_vector(9 downto 0);
    ProgPtrB           : out std_logic_vector(9 downto 0);
    PatMemPtr          : out std_logic_vector(10 downto 0);  -- PATTERN POINTER OUTPUT TO THE PATTERN MEMORY          
    PatStackWriteData  : out std_logic_vector(10 downto 0);
    PatStackReadData   : in  std_logic_vector(10 downto 0);
    SeqControl         : out std_logic_vector(31 downto 18);
    LASReadData        : in  std_logic_vector(11 downto 0);
    LASWriteData       : out std_logic_vector(11 downto 0);
    LSPtr              : out std_logic_vector(5 downto 0);
    LSWrite            : out std_logic;
    LCSReadData        : in  std_logic_vector(15 downto 0);
    LCSWriteDAta       : out std_logic_vector(15 downto 0);
    AddrStackPtr       : out std_logic_vector(5 downto 0);
    AddrStackReadData  : in  std_logic_vector(11 downto 0);
    AddrStackWriteData : out std_logic_vector(11 downto 0);
    AddrStackWrite     : out std_logic;
    SeqBusReq          : out std_logic;
    SeqBusAck          : in  std_logic;
    StatusReg          : out std_logic_vector(15 downto 0)--;
--    LocalSel           : out std_logic
    );

end DecodeExecute;

architecture DecodeExecute_arch of DecodeExecute is

  constant LPP : std_logic_vector(3 downto 0) := "0001";
  constant IPP : std_logic_vector(3 downto 0) := "0010";
  constant DPP : std_logic_vector(3 downto 0) := "0011";
  constant LDA : std_logic_vector(3 downto 0) := "0100";
  constant LMR : std_logic_vector(3 downto 0) := "0101";
  constant LSR : std_logic_vector(3 downto 0) := "0110";

  constant CAL : std_logic_vector(3 downto 0) := "0111";
  constant RET : std_logic_vector(3 downto 0) := "1000";
  constant JCB : std_logic_vector(3 downto 0) := "1001";
  constant LRB : std_logic_vector(3 downto 0) := "1010";
  constant LPB : std_logic_vector(3 downto 0) := "1011";
  constant LPE : std_logic_vector(3 downto 0) := "1100";

  constant DMS : std_logic_vector(3 downto 0) := "1101";
  constant DUS : std_logic_vector(3 downto 0) := "1111";
  constant DSC : std_logic_vector(3 downto 0) := "1110";
  constant NOP : std_logic_vector(3 downto 0) := "0000";

  constant ZERO10 : std_logic_vector(9 downto 0)  := "0000000000";
  constant ZERO12 : std_logic_vector(11 downto 0) := x"000";
  constant ZERO15 : std_logic_vector(15 downto 1) := "000000000000000";

  constant ONE    : std_logic_vector(11 downto 0) := "000000000001";
--    constant ONE11  : std_logic_vector  (10 downto 0) := "00000000001";
  constant TWO    : std_logic_vector(11 downto 0) := "000000000010";
  constant THREE  : std_logic_vector(11 downto 0) := "000000000011";
  constant FOUR   : std_logic_vector(11 downto 0) := "000000000100";
  constant FIVE   : std_logic_vector(11 downto 0) := "000000000101";
  constant SIX    : std_logic_vector(11 downto 0) := "000000000110";
  constant ONE_K  : std_logic_vector(9 downto 0)  := "1111101000";
  constant ONE_uS : std_logic_vector(7 downto 0)  := "00010100";



  signal ZeroClkCtr  : std_logic;
  signal ZerouSecCtr : std_logic;
  signal ZeromsecCtr : std_logic;
  signal ZeroSecCtr  : std_logic;
  signal Run         : std_logic;

  signal OpCode : std_logic_vector (19 downto 0);  -- OPCODE & OPERAND INPUT FROM pipe MODULE           

  signal SecCtr  : std_logic_vector (11 downto 0);  -- DELAY COUNTER
  signal mSecCtr : std_logic_vector (9 downto 0);   -- DELAY COUNTER
  signal uSecCtr : std_logic_vector (9 downto 0);   -- DELAY COUNTER
  signal ClkCtr  : std_logic_vector (7 downto 0);   -- DELAY COUNTER

  signal LoopCtr    : std_logic_vector (15 downto 0);  -- LOOP COUNTER
  signal LoopReg    : std_logic_vector (15 downto 0);  -- LOOP COUNTER
  signal ModeReg    : std_logic_vector (1 downto 0);   -- SEQ_MODE REGISTER
  signal DevAddrReg : std_logic_vector (4 downto 0);   -- SEQ_MODE REGISTER
  signal SelReg     : std_logic_vector (6 downto 0);   -- Board SELECT REGISTER

  signal ProgCtr      : std_logic_vector (11 downto 0);
  signal ProgCtrONE   : std_logic_vector (11 downto 0);
  signal ProgCtrTWO   : std_logic_vector (11 downto 0);
  signal ProgCtrTHREE : std_logic_vector (11 downto 0);
  signal ProgCtrFOUR  : std_logic_vector (11 downto 0);
  signal ProgCtrFIVE  : std_logic_vector (11 downto 0);

  signal JCBFlag    : std_logic;
  signal DontWait   : std_logic;
  signal seqclk_reg : std_logic;

  signal PatMemPtr_reg    : std_logic_vector(10 downto 0);
  signal LSPtr_reg        : std_logic_vector(5 downto 0);
  signal AddrStackPtr_reg : std_logic_vector(5 downto 0);
  signal LocalSel_reg     : std_logic;
  signal SeqBusReq_temp   : std_logic;

begin

  -- detect the rising edge of the slower seqclk
  -- by sampling with the fater sysclk.

  edge_detect : process (sysclk)
  begin
    if rising_edge(sysclk) then
      seqclk_reg <= seqclk;
    end if;
  end process edge_detect;


  SeqControl(31 downto 18) <= (ModeReg & DevAddrReg & SelReg);

-------------------------------------------------------------------------------
-- ACKNOWLEDGE AND REQUEST LOGIC:
-------------------------------------------------------------------------------
  SeqBusReq_temp  <= '0' when (SelReg = "0000000") else '1';
  SeqBusReq       <= SeqBusReq_temp;
  SeqEnablesWrite <= LocalSel_reg and ModeReg(1) and ModeReg(0);  -- Write to the enables register decoded here.  
  Run             <= SeqRunFlag and ((not SeqBusReq_temp) or (SeqBusReq_temp and SeqBusAck)) and DontWait;  -- Principle sequencer enabling flag.
  StatusReg       <= AddrStackPtr_reg(5 downto 0) & ProgCtr(9 downto 0);

-------------------------------------------------------------------------------
-- OPCODE DEMULTIPLEXOR
-------------------------------------------------------------------------------
  process(ProgCtr(1 downto 0), OpCodeL, OpCodeH)  -- this is async OK
  begin
    case ProgCtr (1 downto 0) is
      when "00" =>
        OPCODE(15 downto 0)  <= OpCodeL;
        OPCODE(19 downto 16) <= OpCodeH(3 downto 0);
      when "01" =>
        OPCODE(11 downto 0)  <= OpCodeL(15 downto 4);
        OPCODE(19 downto 12) <= OpCodeH(7 downto 0);
      when "10" =>
        OPCODE(7 downto 0)  <= OpCodeL(15 downto 8);
        OPCODE(19 downto 8) <= OpCodeH(11 downto 0);
      when "11" =>
        OPCODE(3 downto 0)  <= OpCodeL(15 downto 12);
        OPCODE(19 downto 4) <= OpCodeH;
      when others => null;
    end case;
  end process;

-------------------------------------------------------------------------------
-- PROGRAM COUNTER MANAGEMENT
-------------------------------------------------------------------------------
  ProgCtrONE   <= ProgCtr + ONE;
  ProgCtrTWO   <= ProgCtr + TWO;
  ProgCtrTHREE <= ProgCtr + THREE;
  ProgCtrFOUR  <= ProgCtr + FOUR;
  ProgCtrFIVE  <= ProgCtr + FIVE;

  ProgPtrA <= ProgCtr(11 downto 2) when (SeqRunFlag = '1') else PANADDR(9 downto 0);
  ProgPtrB <= ProgCtrFOUR(11 downto 2);

------------------------------------
-- PROGRAM POINTER INCREMENT PROCESS
------------------------------------

  process(sysclk)
  begin
    if rising_edge(sysclk) then
      if (RESET = '1') then
        ProgCtr <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE(3 downto 0) is
            when IPP | DPP | NOP | LPB =>
              ProgCtr <= ProgCtrONE;
            when LMR | LRB =>
              ProgCtr <= ProgCtrTWO;
            when DMS | DUS | DSC | LSR | LDA =>
              ProgCtr <= ProgCtrTHREE;
            when LPP =>
              ProgCtr <= ProgCtrFOUR;
            when CAL =>
              ProgCtr <= OpCode(15 downto 4);
            when RET =>
              ProgCtr <= AddrStackReadData;
            when JCB =>
              if (JCBFlag = '1') then
                ProgCtr <= OpCode(15 downto 4);
              else
                ProgCtr <= ProgCtrFIVE;
              end if;
            when LPE =>
              if (LoopCtr(15 downto 1) = ZERO15) then
                ProgCtr <= ProgCtrONE;
              else
                ProgCtr <= LASReadData;
              end if;
            when others => null;
          end case;
        elsif (SeqRunFlag = '0') then
          ProgCtr <= (others => '0');  -- Initialize program counter to zero when not running.
        end if;
      end if;
    end if;
  end process;

--------------------------------
-- PROGRAM POINTER STACK PROCESS
--------------------------------

  process(sysclk, RESET)
  begin
    if rising_edge(sysclk) then
      if (RESET = '1') then
        AddrStackPtr_reg   <= (others => '0');
        AddrStackWriteData <= (others => '0');
        AddrStackWrite     <= '0';
      elsif (seqclk = '1' and seqclk_reg = '0') then
        AddrStackWrite <= '0';
        if (Run = '1') then
          case OPCODE (3 downto 0) is
            when CAL =>
              AddrStackPtr_reg   <= AddrStackPtr_reg + '1';
              AddrStackWriteData <= ProgCtrFOUR;
              AddrStackWrite     <= '1';
            when RET =>
              AddrStackPtr_reg <= AddrStackPtr_reg - '1';
            when others => null;
          end case;
        elsif (SeqRunFlag = '0') then
          AddrStackPtr_reg <= (others => '0');
          AddrStackWrite   <= '0';
        end if;
      end if;
    end if;
  end process;

  AddrStackPtr <= AddrStackPtr_reg;

-------------------------------------------------------------------------------
-- PATTERN POINTER MANAGEMENT
-------------------------------------------------------------------------------

  process(sysclk)
  begin
    if rising_edge(sysclk) then

      if (RESET = '1') then
        PatMemPtr_reg     <= (others => '0');
        PatStackWriteData <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE (3 downto 0) is
            when IPP =>
              PatMemPtr_reg <= PatMemPtr_reg + 1;
            when DPP =>
              PatMemPtr_reg <= PatMemPtr_reg - 1;
            when LPP =>
              PatMemPtr_reg <= OpCode(14 downto 4);
            when CAL =>
              PatStackWriteData <= PatMemPtr_reg;
            when RET =>
              PatMemPtr_reg <= PatStackReadData;
            when others => null;
          end case;
        elsif (SeqRunFlag = '0') then
          PatMemPtr_reg <= (others => '0');  -- Initialize pattern pointer to 0 when not running.
        end if;
      end if;
    end if;
  end process;

  PatMemPtr <= PatMemPtr_reg;

-------------------------------------------------------------------------------
-- SEQ BUS MODE WORD MANAGEMENT 
-------------------------------------------------------------------------------
  process(sysclk)
  begin
    if rising_edge(sysclk) then
      if (RESET = '1') then
        ModeReg <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE (3 downto 0) is
            when LMR =>
              ModeReg <= OPCODE (5 downto 4);
            when others =>
              ModeReg <= ModeReg;
          end case;
        elsif (SeqRunFlag = '0') then
          ModeReg <= (others => '0');  -- Initialize Mode register to RESET command
        end if;
      end if;
    end if;
  end process;

-------------------------------------------------------------------------------
-- SEQ BUS DEVICE ADDRESS MANAGMENT
-------------------------------------------------------------------------------
  process(sysclk)
  begin
    if rising_edge(sysclk) then

      if (RESET = '1') then
        DevAddrReg <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE (3 downto 0) is
            when LDA =>
              DevAddrReg <= OPCODE (8 downto 4);
            when others =>
              DevAddrReg <= DevAddrReg;
          end case;
        elsif (SeqRunFlag = '0') then
          DevAddrReg <= (others => '0');  -- Initialize Device Addr word to zero.
        end if;
      end if;
    end if;
  end process;

-------------------------------------------------------------------------------
-- SEQ BUS BOARD SELECT MANAGEMENT
-------------------------------------------------------------------------------
  process(sysclk)
  begin
    if rising_edge(sysclk) then
      if (RESET = '1') then
        SelReg       <= (others => '0');
        LocalSel_reg <= '0';
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE(3 downto 0) is
            when LSR =>
              SelReg       <= OPCODE(11 downto 5);
              LocalSel_reg <= OPCODE(4);
            when others =>
              if (AutoCancelFlag = '1') then
                SelReg       <= (others => '0');
                LocalSel_reg <= '0';
              else
                SelReg       <= SelReg;
                LocalSel_reg <= LocalSel_reg;
              end if;
          end case;
        elsif (SeqRunFlag = '0') then
          SelReg       <= (others => '0');
          LocalSel_reg <= '0';
        end if;
      end if;
    end if;
  end process;

--  LocalSel <= LocalSel_reg;

-------------------------------------------------------------------------------
-- CONDITIONAL JUMP FLAG MULTIPLEXOR
-------------------------------------------------------------------------------
  process(CmdReg, OpCode (19 downto 16))
  begin
    case OpCode (19 downto 16) is
      when "0000" => JCBFlag <= CmdReg(0);
      when "0001" => JCBFlag <= CmdReg(1);
      when "0010" => JCBFlag <= CmdReg(2);
      when "0011" => JCBFlag <= CmdReg(3);
      when "0100" => JCBFlag <= CmdReg(4);
      when "0101" => JCBFlag <= CmdReg(5);
      when "0110" => JCBFlag <= CmdReg(6);
      when "0111" => JCBFlag <= CmdReg(7);
      when "1000" => JCBFlag <= CmdReg(8);
      when "1001" => JCBFlag <= CmdReg(9);
      when "1010" => JCBFlag <= CmdReg(10);
      when "1011" => JCBFlag <= CmdReg(11);
      when "1100" => JCBFlag <= CmdReg(12);
      when "1101" => JCBFlag <= CmdReg(13);
      when "1110" => JCBFlag <= CmdReg(14);
      when "1111" => JCBFlag <= CmdReg(15);
      when others => null;
    end case;
  end process;

-------------------------------------------------------------------------------
-- LOOP COUNTER AND REGISTER MANAGEMENT
-------------------------------------------------------------------------------
  process(sysclk)
  begin
    if rising_edge(sysclk) then

      if (RESET = '1') then
        LoopCtr <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE(3 downto 0) is
            when LRB =>
              LoopCtr <= LoopReg;  -- 16 bit loop counter register contents   
            when LPB =>
              LoopCtr <= PatternData;  -- 16 bit immediate loop count from pattern memory data
            when LPE =>
              if (LoopCtr(15 downto 1) = ZERO15) then
                LoopCtr <= LCSReadData;
              else
                LoopCtr <= LoopCtr - 1;
              end if;
            when others => null;
          end case;
        elsif (SeqRunFlag = '0') then
          LoopCtr <= (others => '0');
        end if;
      end if;
    end if;
  end process;

------------------------------------------------        
-- LOOP COUNTER STACK POINTER MANAGEMENT PROCESS
------------------------------------------------

  process(sysclk)
  begin
    if rising_edge(sysclk) then

      if (Reset = '1') then
        LSPtr_reg    <= (others => '0');
        LSWrite      <= '0';
        LASWriteData <= (others => '0');
        LCSWriteData <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        LSWrite <= '0';
        if (Run = '1') then
          case OPCODE(3 downto 0) is
            when LRB =>
              LSWrite      <= '1';
              LSPtr_reg    <= LSPtr_reg + 1;
              LASWriteData <= ProgCtrTWO;
              LCSWriteData <= LoopCtr;
            when LPB =>
              LSWrite      <= '1';
              LSPtr_reg    <= LSPtr_reg + 1;
              LASWriteData <= ProgCtrONE;
              LCSWriteData <= LoopCtr;
            when LPE =>
              if (LoopCtr(15 downto 1) = ZERO15) then
                LSPtr_reg <= LSPtr_reg - 1;
              end if;
            when others => null;
          end case;
        elsif (SeqRunFlag = '0') then
          LSPtr_reg <= (others => '0');
          LSWrite   <= '0';
        end if;
      end if;
    end if;
  end process;

  LSPtr <= LSPtr_reg;

--------------------------------------
-- LOOP COUNTER REGISTER DEMULTIPLEXOR
--------------------------------------

  loopreg <= loopreg0 when (opcode(7 downto 4) = "0000") else
             loopreg1  when (opcode(7 downto 4) = "0001") else
             loopreg2  when (opcode(7 downto 4) = "0010") else
             loopreg3  when (opcode(7 downto 4) = "0011") else
             loopreg4  when (opcode(7 downto 4) = "0100") else
             loopreg5  when (opcode(7 downto 4) = "0101") else
             loopreg6  when (opcode(7 downto 4) = "0110") else
             loopreg7  when (opcode(7 downto 4) = "0111") else
             loopreg8  when (opcode(7 downto 4) = "1000") else
             loopreg9  when (opcode(7 downto 4) = "1001") else
             loopreg10 when (opcode(7 downto 4) = "1010") else
             loopreg11 when (opcode(7 downto 4) = "1011") else
             loopreg12 when (opcode(7 downto 4) = "1100") else
             loopreg13 when (opcode(7 downto 4) = "1101") else
             loopreg14 when (opcode(7 downto 4) = "1110") else
             loopreg15 when (opcode(7 downto 4) = "1111") else
             (others => '0');

-------------------------------------------------------------------------------
-- DELAY COUNTERS
-------------------------------------------------------------------------------
  process(sysclk)
  begin
    if rising_edge (sysclk) then

      if (RESET = '1' or SeqRunFlag = '0') then
        ClkCtr  <= (others => '0');
        uSecCtr <= (others => '0');
        msecCtr <= (others => '0');
        SecCtr  <= (others => '0');
      elsif (seqclk = '1' and seqclk_reg = '0') then
        if (Run = '1') then
          case OPCODE(3 downto 0) is
            when DMS =>
              mSecCtr(7 downto 0) <= Opcode(11 downto 4);
            when DUS =>
              uSecCtr(7 downto 0) <= Opcode(11 downto 4);
            when DSC =>
              ClkCtr(7 downto 0) <= Opcode(11 downto 4);
            when others => null;
          end case;
        end if;
        if (ZeroClkCtr = '0') then
          ClkCtr <= ClkCtr - 1;
        elsif (ZerouSecCtr = '0') then
          uSecCtr <= uSecCtr - 1;
          ClkCtr  <= ONE_uS;
        elsif (ZeromsecCtr = '0') then
          mSecCtr <= mSecCtr - 1;
          uSecCtr <= ONE_K;
          ClkCtr  <= ONE_uS;
        elsif (ZeroSecCtr = '0') then
          SecCtr  <= SecCtr - 1;
          mSecCtr <= ONE_K;
          uSecCtr <= ONE_K;
          ClkCtr  <= ONE_uS;
        end if;
      end if;
    end if;
  end process;

  ZeroClkCtr  <= '1' when (ClkCtr = ZERO10)  else '0';
  ZerouSecCtr <= '1' when (uSecCtr = ZERO10) else '0';
  ZeromsecCtr <= '1' when (mSecCtr = ZERO10) else '0';
  ZeroSecCtr  <= '1' when (SecCtr = ZERO12)  else '0';
  DontWait    <= ZeroClkCtr and ZerouSecCtr and ZeromsecCtr and ZeroSecCtr;

end DecodeExecute_arch;
