------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   PAN COMMAND DECODER
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   PAN_COMMAND_DECODE.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/22/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :  RECEIVES 32 BIT WORDS FROM THE FPGA INTERFACE, DECODES
-- THE DHE COMMAND AND ASSEMBLES THE DATA STRUCTURE FOR TRANSMISSION ON 
-- THE SEQUENCER BUS
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
-- 02/26/2003  PCM   2.1  ORIGINAL VERSION
-- 05/07/2003  KAVI  2.1  MODIFIED TO INCLUDE LOCAL BOARD OPERATIONS
-- 07/08/2003  PCM   2.2  ADDED RESET DECODING LOGIC
-- 04/22/2004  pcm   4.0  ADAPTED FOR HARDWARE REVISION A.
-- 12/10/2008  obb        Changed port DATA_IN_RD from inout to out, as it not
--                        accepts input data.
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK          : SYSTEM CLOCK
-- RST          : SYSTEM RESET
-- DATA_IN_VALID: FLAG FROM FPGA INTERFACE TO SIGNAL DATA AVAILABLE
-- DATA_IN_RD   : READ STROBE BACK TO THE FPGA INTERFACE.
-- DATA_IN      : 32 BIT RAW DATA FROM THE FPGA INTERFACE
-- MODE         : DECODED MODE WORD FROM DHE DATA
-- DEVADDR      : DECODED DEV ADDRESS FROM DHE DATA
-- ADDRDATA     : DECODED DATA / ADDRESS FROM DHE DATA
-- SEQ_BUS_REQ  : THIS MODULES REQUEST FOR ACCESS TO TEH SEQUENCER BUS 
-- SEQ_BUS_ACK  : SEQUENCER BUS MULTIPLEXOR BUS GRANT SIGNAL
-- BOARD_SELECT : SEPERATE BOARD SELECT SIGNALS FROM DHE DATA
-- PIX_FIFO_WRT: WRITE CODE TO THE FIBER FPGA PIXEL BUS FIFO PORT
-- START_EXP    : START EXPOSURE STROBE TO SEQUENCER MODULE
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

use work.constantsPkg.all;

-------------------------------------------------------------------------------
-- TODO: We are not using neither SOFT_RST nor HARD_RST ports. I delete this
-- ports and all logic behind. I will add this features on Personality Module,
-- where, right now we have more or less the same functionality.
-------------------------------------------------------------------------------

entity Pan_Command_Decode is
  port (CLK           : in  std_logic;
        RESET         : in  std_logic;
        DATA_IN_VALID : in  std_logic;
        DATA_IN_RD    : out std_logic;
        DATA_IN       : in  std_logic_vector(31 downto 0);
        ECHO_DATA     : out std_logic_vector(31 downto 0);
        MODE          : out std_logic_vector(1 downto 0);
        DEVADDR       : out std_logic_vector(4 downto 0);
        ADDRDATA      : out std_logic_vector(31 downto 0);
        LOCDATA       : out std_logic_vector(31 downto 0);
        LOCADDR       : out std_logic_vector(15 downto 0);
        LOCSEL        : out std_logic;
        SEQ_BUS_REQ   : out std_logic;
        SEQ_BUS_ACK   : in  std_logic;
        BOARD_SELECT  : out std_logic_vector(7 downto 0);
        PIXFIFO_WRT   : out std_logic;
        START_EXP     : out std_logic;
        ECHO_REQ      : out std_logic;
        WRT           : out std_logic;
        RD            : out std_logic;
        WRT_PROG      : out std_logic;
        seqWrite_n    : out std_logic;
        exp2cntr : out std_logic;
        ASYNC_PENDING : out std_logic
        );
end Pan_Command_Decode;

architecture Behavioral of Pan_Command_Decode is

------------------------------------------------------------------------
-- CONSTANTS FOR PAN COMMAND CODES IN FIRST WORD (BITS 31 -> 30)
------------------------------------------------------------------------
  constant DHE_EXP  : std_logic_vector(1 downto 0) := "00";
  constant DHE_READ : std_logic_vector(1 downto 0) := "01";
  constant DHE_WRT  : std_logic_vector(1 downto 0) := "10";
  constant DHE_ASYN : std_logic_vector(1 downto 0) := "11";

------------------------------------------------------------------------
-- CONSTANTS FOR SEQUENCER BUS MODE WORD
------------------------------------------------------------------------
  constant SEQ_RD   : std_logic_vector(1 downto 0) := "01";
  constant SEQ_WR16 : std_logic_vector(1 downto 0) := "10";
  constant SEQ_WR32 : std_logic_vector(1 downto 0) := "11";


------------------------------------------------------------------------
-- LOCAL VARIABLES
------------------------------------------------------------------------
  constant RD_ECHO_DELAY : std_logic_vector(3 downto 0) := "1000";  -- Number of clocks to delay for a read.

  type statetype is (waitAsync,
                     idle,
                     exp_1,
                     asyn_1,
                     rd_1,
                     rd_2,
                     RdEchoWait,
                     wr_1,
                     wr_2,
                     wr_3,
                     fin);

  signal state           : statetype;
  signal dhe_cmd         : std_logic_vector(1 downto 0);
  signal dhe_addr        : std_logic_vector(15 downto 0);
  signal dhe_board       : std_logic_vector(7 downto 0);
  signal addr_buffr      : std_logic_vector(15 downto 0);
  signal dhe_board_latch : std_logic_vector(7 downto 0);
  signal RdEchoCntr      : std_logic_vector(3 downto 0);
  signal wrt_32          : std_logic;
  signal wrt_32_flag     : std_logic;
  signal locwrt          : std_logic;
  signal EchoEnable      : std_logic;
  signal START_EXP_s     : std_logic;
  signal AsynRcvd        : std_logic;

  signal intMode        : std_logic_vector(1 downto 0);
  signal intDevAddr     : std_logic_vector(4 downto 0);
  signal intAddrData    : std_logic_vector(31 downto 0);
  signal intBoardSelect : std_logic_vector(7 downto 0);

  signal intAsyncPending : std_logic;

  signal intAckLinkData : std_logic;    -- its purpouse is feed DATA_IN_RD
  signal intLocSel : std_logic;

  signal pixFifo_Wrt_n : std_logic;
begin

  exp2cntr <= start_exp_s;
  -----------------------------------------------------------------------------
  -- SeqWrite_n (former fpga_ctl_1)
  -----------------------------------------------------------------------------
  seqwrite_n_proc: process (clk, reset)
  begin  -- process seqwrite_n_proc
    if reset = '1' then                 -- asynchronous reset (active high)
      seqWrite_n <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      seqWrite_n <= intLocSel or pixFifo_Wrt_n;
    end if;
  end process seqwrite_n_proc;
------------------------------------------------------------------------
-- Static assignments
------------------------------------------------------------------------
  MODE         <= intMode;
  DEVADDR      <= intDevAddr;
  ADDRDATA     <= intAddrData;
  BOARD_SELECT <= intBoardSelect;

  LOCSEL <= intLocSel;

  PixFifo_Wrt <= not pixFifo_Wrt_n;

  ASYNC_PENDING <= intAsyncPending;

  DATA_IN_RD <= intAckLinkData;

  dhe_cmd   <= DATA_IN(31 downto 30);   -- Convenience only !
  dhe_addr  <= DATA_IN (15 downto 0);
  dhe_board <= DATA_IN(23 downto 16);

  ECHO_REQ <= EchoEnable and intAckLinkData;

  wrt_32 <= '1' when dhe_addr(15 downto 5) = x"00" & "00" else
            '0';

  -- dhe_cmd = DHE_ASYN when DATA_IN(31:30) = "11"
  AsynRcvd <= DATA_IN(31) and DATA_IN(30);
--  AsynRcvd <= '1' when dhe_cmd = DHE_ASYN else
--              '0';


  -----------------------------------------------------------------------------
  -- Acknowledge to slink we have read data received
  -----------------------------------------------------------------------------
  ackDataRead : process (clk, reset)
  begin  -- process ackDataRead
    if reset = '1' then                 -- asynchronous reset (active high)
      intAckLinkData <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = waitAsync and DATA_IN_VALID = '1' then
        intAckLinkData <= '1';
      elsif state = idle and DATA_IN_VALID = '1' then
        intAckLinkData <= '1';
      elsif state = wr_2 and DATA_IN_VALID = '1' then
        intAckLinkData <= '1';
      else
        intAckLinkData <= '0';
      end if;
    end if;
  end process ackDataRead;
  -----------------------------------------------------------------------------
  -- AsyncPending signal
  -----------------------------------------------------------------------------
  intAsyncPending <= '1' when state = waitAsync else
                     '0';

  -----------------------------------------------------------------------------
  -- Echo Enable and echoData
  -----------------------------------------------------------------------------
  echoEnableReg : process (clk, reset)
  begin  -- process echoEnableReg
    if reset = '1' then                 -- asynchronous reset (active high)
      EchoEnable <= '1';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = idle then
        if DATA_IN_VALID = '1' and dhe_cmd = DHE_EXP then
          EchoEnable <= '0';
        else
          EchoEnable <= '1';
        end if;
      end if;
    end if;
  end process echoEnableReg;
  -- purpose: Generate data to echo
  -- type   : sequential
  -- inputs : clk, reset
  -- outputs: 
  echoDataReg : process (clk, reset)
  begin  -- process echoDataReg
    if reset = '1' then                 -- asynchronous reset (active high)
      ECHO_DATA <= (others => '0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = idle then
        ECHO_DATA <= DATA_IN;
      elsif state = waitAsync then
        if AsynRcvd = '0' then
          ECHO_DATA <= x"C00100FF";
        else
          ECHO_DATA <= DATA_IN;
        end if;
      elsif state = wr_2 then
        ECHO_DATA <= DATA_IN;
      end if;
    end if;
  end process echoDataReg;

  -----------------------------------------------------------------------------
  -- locwrt
  -----------------------------------------------------------------------------
  locwrtGen : process (clk, reset)
  begin  -- process locwrtGen
    if reset = '1' then                 -- asynchronous reset (active low)
      locwrt <= '0';
    elsif CLK'event and CLK = '1' then  -- rising clock edge
      if state = wr_2 and DATA_IN_VALID = '1' then
        locwrt <= '1';
      elsif state = wr_3 and (SEQ_BUS_ACK = '1' or intBoardSelect(0) = '1') then
        locwrt <= '0';
      elsif state = fin then
        locwrt <= '0';
      end if;
    end if;
  end process locwrtGen;

  -----------------------------------------------------------------------------
  -- intBoardSelect generation
  -----------------------------------------------------------------------------
  intBoardSelectGenproc : process (clk, reset)
  begin  -- process intBoardSelectGenproc
    if reset = '1' then                     -- asynchronous reset (active low)
      intBoardSelect <= (others => '0');
    elsif clk'event and clk = '1' then      -- rising clock edge
      if state = idle then
        intBoardSelect <= (others => '0');
      elsif state = RdEchoWait and (RdEchoCntr >= RD_ECHO_DELAY) then
        intBoardSelect <= dhe_board_latch;
      elsif state = wr_2 and DATA_IN_VALID = '1' then
        intBoardSelect <= dhe_board_latch;
      elsif state = wr_3 and (SEQ_BUS_ACK = '1' or intBoardSelect(0) = '1') then
        intBoardSelect <= (others => '0');  -- Kill the board select line(s)
      elsif state = fin then
        intBoardSelect <= (others => '0');
      end if;
    end if;
  end process intBoardSelectGenproc;

  -----------------------------------------------------------------------------
  -- PixFifo_Wrt_n
  -----------------------------------------------------------------------------
  pixfifowrtGenProc : process (clk, reset)
  begin  -- process pixfifowrtGenProc
    if reset = '1' then                 -- asynchronous reset (active low)
      PixFifo_Wrt_n <= '1';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = rd_2 then
        PixFifo_Wrt_n <= '0';           -- Signal the pixel bus fifo on the
                                        -- fiber fpga to pick up the data.
      elsif state = fin then
        PixFifo_Wrt_n <= '1';
      end if;
    end if;
  end process pixfifowrtGenProc;
  
  -----------------------------------------------------------------------------
  -- Start_exp_s
  -----------------------------------------------------------------------------
--   Probably could be changed to:
--  start_exp_s <= '1' when state = exp_1 else
--                 '0';
  startExp_sGenProc : process (clk, reset)
  begin  -- process startExp_sGenProc
    if reset = '1' then                 -- asynchronous reset (active low)
      start_exp_s <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = idle then
        if DATA_IN_VALID = '1' and intAckLinkData = '0' then
          if dhe_cmd = DHE_EXP then
            start_exp_s <= '1';         -- set the strobe to true
          end if;
        end if;
      elsif state = fin then
        start_exp_s <= '0';
      end if;
    end if;
  end process startExp_sGenProc;

  -----------------------------------------------------------------------------
  -- intAddrDataGenProc
  -----------------------------------------------------------------------------
  intAddrDataGenProc : process (clk, reset)
  begin  -- process intAddrDataGenProc
    if reset = '1' then                 -- asynchronous reset (active low)
      intAddrData <= (others => '0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if state = idle and DATA_IN_VALID = '1' then
        if dhe_cmd = DHE_EXP then
          intAddrData <= x"0000" & dhe_addr;
        elsif dhe_cmd = DHE_READ then
          intAddrData <= dhe_addr(15 downto 0) & x"0000";
        end if;
      elsif state = wr_2 and DATA_IN_VALID = '1' then
        if wrt_32_flag = '1' then
          intAddrData <= DATA_IN;
        else
          intAddrData <= addr_buffr(15 downto 0) & DATA_IN(15 downto 0);
        end if;
      end if;
    end if;
  end process intAddrDataGenProc;


------------------------------------------------------------------------
-- Decoder state machine
------------------------------------------------------------------------

  cmd_decode : process (CLK, reset)
  begin
    if reset = '1' then
      state       <= waitAsync;
      SEQ_BUS_REQ <= '0';
    elsif clk'event and clk = '1' then
      case state is
        when waitAsync =>
          if DATA_IN_VALID = '1' and AsynRcvd = '1' then
            state <= asyn_1;
          end if;
        when idle =>
          if ((DATA_IN_VALID and (not intAckLinkData)) = '1') then
            dhe_board_latch <= dhe_board;
            case dhe_cmd is
              when DHE_EXP =>           -- Start exposure command from DHE
                state <= exp_1;         -- used to set sequencer mpu flag
              when DHE_READ =>          -- Read command from DHE
                intMode    <= SEQ_RD;   -- Correct mode for a read opperation
                intDevAddr <= (others => '0');
                RdEchoCntr <= (others => '0');  -- Reset the clock counter
                state      <= RdEchoWait;
              when DHE_WRT =>  -- Write command from DHE, wait for the data rdy
                               -- strobe to go false
                wrt_32_flag <= wrt_32;  -- 1 bit memory to retain mode for next
                                        -- state
                if wrt_32 = '1' then
                  intMode    <= SEQ_WR32;  -- Correct mode for a 32 bit write
                  intDevAddr <= dhe_addr(4 downto 0);
                else
                  intMode    <= SEQ_WR16;  -- Correct mode for a 16 bit write
                  addr_buffr <= dhe_addr;  -- Catch the write address for later
                  intDevAddr <= (others => '0');
                end if;
                state <= wr_1;
              when DHE_ASYN =>          -- Async message acknowledge;
                state <= asyn_1;
              when others =>            -- Errr !
                state <= idle;
            end case;
          else
            State <= Idle;
          end if;
        when asyn_1 =>                  -- dummy state to cancel the read flag
          state <= fin;
        when exp_1 =>                   -- Start exposure command here
          state <= fin;
        when RdEchoWait =>  -- Wait here for the echo to be transfered and sent
                            -- via the pixel fpga first.
          if (RdEchoCntr < RD_ECHO_DELAY) then
            RdEchoCntr <= RdEchoCntr + 1;
            state      <= RdEchoWait;
          else
            SEQ_BUS_REQ <= '1';         -- Assert the bus request flag
            state       <= rd_1;        -- Setup the next state
          end if;
        when rd_1 =>
          if ((SEQ_BUS_ACK = '1') or (intBoardSelect(0) = '1')) then
            state <= rd_2;
          end if;
        when rd_2 =>  -- One clock cycle later, pulse the write fifo control
                      -- lines 
          state <= fin;                 -- Go back to idle state.
        when wr_1 =>
          state <= wr_2;    -- Wait state for data ready flag to clear.
        when wr_2 =>
          if (DATA_IN_VALID = '1') then
            SEQ_BUS_REQ <= '1';
            state       <= wr_3;
          end if;
        when wr_3 =>
          if ((SEQ_BUS_ACK = '1') or (intBoardSelect(0) = '1')) then
            SEQ_BUS_REQ <= '0';
            state       <= fin;         -- Go back to idle state.
          end if;
        when fin =>
          SEQ_BUS_REQ <= '0';
          State       <= idle;
        when others =>
          state <= idle;
      end case;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Locsel register (I guess he does this to delay it one cycle)
  -----------------------------------------------------------------------------
  locselgen: process (clk, reset)
  begin  -- process locselgen
    if reset = '1' then                 -- asynchronous reset (active high)
      intLocSel <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      intLocSel <= intBoardSelect(0);
    end if;
  end process locselgen;

  -----------------------------------------------------------------------------
  -- decode logic?
  -----------------------------------------------------------------------------

  decode : process (CLK)
  begin
    if (rising_edge(CLK)) then
      WRT_PROG  <= '0';
      WRT       <= '0';
      RD        <= '0';
      START_EXP <= '0';
      if (intBoardSelect(0) = '1') then
        case intMode is
          when SEQ_RD =>
            LOCADDR <= intAddrData(31 downto 16);
            WRT     <= '0';
            RD      <= '1';
          when SEQ_WR16 =>
            LOCDATA <= x"0000" & intAddrData(15 downto 0);
            LOCADDR <= intAddrData(31 downto 16);
            if (intAddrData(31 downto 26) = "010000") then
              WRT_PROG <= locwrt;
            end if;
            WRT <= locwrt;
          when SEQ_WR32 =>
            LOCDATA <= intAddrData;
            LOCADDR <= "00000000000" & intDevAddr;
            WRT     <= locwrt;
          when others => null;
                         LOCDATA <= intAddrData;
                         WRT     <= '0';
        end case;
      elsif (START_EXP_s = '1') then
        LOCDATA   <= intAddrData;
        START_EXP <= '1';
      end if;
    end if;
  end process;

end Behavioral;
