------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MCB
-- DEVICE ID    :   MCB SEQUENCER FPGA
-- PARENT MODULE:   TOP LEVEL DESIGN  
-- MODULE       :   PERSONALITY MODULE
-- VERSION      :   4.4 
------------------------------------------------------------------------
-- FILE NAME    :   PERSONALITYMODULE.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   02/14/2005
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE Webpack 6.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- THIS MODULE PROVIDES THE LOGIC MECHANISM TO INTERFACE THE SEQUENCER
-- BUS INTERFACE TO ANY PARTICULAR VIRTEX FPGA DESIGN. 
-- IT PROVIDES FOR LOCAL ADDRESS DECODE LOGIC AND A SIMPLE DATA SELECTION
-- MUX TO PERSONALIZE YOUR DESIGN TO A PARTICULAR FPGA LOGIC IMPLEMENTATION 
-- TO THE SEQUENCER INTERFACE LOGIC BLOCK.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- FIXED ADDRESS ASSIGNMENTS
-- LED INDICATOR SELECT PORT     = 0xFFF9 (3 BITS WRITE - 3 BITS READ)
-- CIRCUIT BOARD SERIAL NUMBER   = 0xFFFA (NO WRITE - 48 BITS READ)
-- CIRCUIT BOARD TEMPERATURE     = 0xFFFB (NO WRITE - 16 BITS READ)
-- CONTROL REGISTER ADDRESS      = 0xFFFC (16 BIT WRITE - 16 BIT READ)
-- STATUS REGISTER ADDRESS       = 0XFFFD (NO WRITE - 16 BIT READ) 
-- BOARD ID REGISTER ADDRESS     = 0xFFFE (NO WRITE - 16 BIT READ)
-- CODE ID REGISTER ADDRESS      = 0xFFFF (NO WRITE - 16 BIT READ)
-- SEQUENCER FPGA SPECIFIC ADDRESS MAP
-- REQUESTED INTEGRATION TIME    = 0x0000 (32 BIT WRITE - 32 BIT READ)
-- TEST DATA GENERATOR           = 0x0001 (32 BITS WRITE - 32 BITS READ)
-- CLOCK ENABLE REGISTER         = 0x0100 (7 BITS WRITE - 16 BITS READ)
-- ACTUAL INTEGRATION TIME       = 0x0101 (NO WRITE - 32 BIT READ)
-- SEQUENCER MPU COMMAND REG     = 0x0102 (4 BITS WRITE - 16 BIT READ)
-- SEQUENCER MPU LOOP REGISTERS  = 0x0110 => 0x011F (16 BITS WRITE - 16 BITS READ)
-- SERIAL CONFIGURATION DATA     = 0x0200 => 0x027F (16 BITS WRITE - 16 BITS READ) (NOTE 1)
-- DHE CLOCK SYNC PHASE CONTROL  = 0x0280 (8 BITS WRITE - 8 BITS READ) (NOTE 1)
-- PIXEL FPGA LED INDICATOR PORT = 0x0281 (3 BIT WRITE - 3 BITS READ) (NOTE 1)
-- SYNC LINK SERIAL DATA PORT    = 0x0282 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- HIGH SPEED SERIAL LINK PORT   = 0x0283 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- PIXEL FPGA STATUS PORT        = 0x0284 (NO WRITE - 16 BITS READ) (NOTE 1, 2)
-- EXTERNAL DATA PORT            = 0x0285 (4 BITS WRITE - 4 BITS READ) (NOTE 1, 2)
-- TEMP / DAC CHANNEL PORT       = 0x02A0 => 0x02A7 (16 BITS WRITE - 16 BITS READ) (NOTE 1, 2)
-- SERIAL CONFIG FETCH COMMAND   = 0x02A8  (3 BITS WRITE - 3 BITS READ - TRIGGERS A READ CYCLE OF EEPROM)
-- TEMP / ADC CHANNEL PORT       = 0x02C0 => 0x02C7 (WRITE (NO SIGNIFICANT DATA - 12 BITS READ) (NOTE 1, 2)
-- TEMP / ADC CHANNEL MODE PORT  = 0x02C8 => 0x02CF (4 BITS WRITE - 4 BITS READ) (NOTE 1, 2)
-- SEQUENCER MPU PATTERN MEMORY  = 0x1000 => 0X1BFF (16 BITS WRITE - 16 BITS READ)
-- SEQUENCER MPU PROGRAM MEMORY  = 0x4000 => 0x43FF (16 BITS WRITE - 16 BITS READ)
-- NOTES:
-- 1. THESE FUNCTIONS ARE PHYSICALLY ETCHED INTO THE PIXEL FPGA LOGIC.
-- 2. NOT YET IMPLEMENETED OR TESTED.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE        AUTHOR  VERSION  MODIFICATIONS
-- 01/30/2003  PCM     0.1      ORIGINAL VERSION
-- ??/??/????  PCM     0.2 => 4.3 VARIOUS UNDOCUMENTED REVISIONS.
-- 02/14/2005  PCM     4.4      CORRECTED PATTERN GENERATOR ENABLE BIT FROM 15 TO BIT 7
--                              SINCE BIT 15 IS IN CONFLICT WITH SEQUENCER MASTER SYNC DELAY VALUE
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- READ       : Enables the tristate buffers to the pixel bus.
-- BOARD_SEL  : Global enable. Signifies that the board has been selected.  
-- CONTROL_REG: Output to seq interface to allow readback of content.
-- ADDRESS    : Address bus input for local decoding.
-- CONTROL_SEL: Output to the control reg in the sequencer interface module.
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use work.constantsPkg.all;

entity Personality_ModuleV45 is
  port (SYSCLK       : in  std_logic;
        reset        : in  std_logic;
        WRITE        : in  std_logic;
        BOARD_SEL    : in  std_logic;
        LED_REG_SEL  : out std_logic;
        crateSyncIn  : in std_logic;
        crateSyncOut : in std_logic;
        LED_DATA     : in  std_logic_vector(1 downto 0);
        TEMP_DATA_IN : in  std_logic_vector(9 downto 0);
        SN_DATA      : in  std_logic_vector(31 downto 0);
        ADDRESS      : in  std_logic_vector(15 downto 0);
        STATUS_REG   : in  std_logic_vector(15 downto 0);
        CLOCKEN_REG  : in  std_logic_vector(15 downto 0);
        SYNCDEL_REG  : in  std_logic_vector(7 downto 0);
        clkDel_reg : in std_logic_vector(7 downto 0);
        TST_CTR_REG  : in  std_logic_vector(31 downto 0);
        SEQ_CMD_REG  : in  std_logic_vector(15 downto 0);
        SEQ_INT_REG  : in  std_logic_vector(31 downto 0);
        SEQ_INT_CTR  : in  std_logic_vector(31 downto 0);
        SEQ_LPP_REG  : in  std_logic_vector(15 downto 0);
        SEQ_PAT_MEM  : in  std_logic_vector(15 downto 0);
        SEQ_PGM_MEM  : in  std_logic_vector(15 downto 0);
        ECHO_DATA_IN : in  std_logic_vector(31 downto 0);
        DATA_IN      : in  std_logic_vector(15 downto 0);
        DATA_OUT     : out std_logic_vector(31 downto 0);
        CONTROL_OUT  : out std_logic_vector(15 downto 0);
        CLOCKEN_SEL  : out std_logic;
        SYNCDEL_SEL  : out std_logic;
        clkDel_sel   : out std_logic;
        program_n    : out std_logic;
        exp2cntr : in std_logic;
        SEQ_CMD_SEL  : out std_logic;
        SEQ_INT_SEL  : out std_logic;
        SEQ_LPP_SEL  : out std_logic;
        SEQ_PAT_SEL  : out std_logic;
        TST_CTR_REQ  : in  std_logic;
        TST_CTR_WRT  : out std_logic;
        TST_CTR_ENB  : out std_logic;
        switches_sel : out std_logic;
        switches_reg : in  std_logic_vector(4 downto 0);
--        TEMP_RD_STB  : out std_logic;
        pixSeqSyncOut     : out std_logic;
        READ         : in  std_logic;
        READ_STROBE  : in  std_logic;
        ECHO_REQ     : in  std_logic;
        seqDataReq   : out std_logic;
        cmdRdy       : out std_logic--;
--        EXPOSE_START : out std_logic;
--        FRAME_START  : out std_logic;
--        MPU_EVENTS   : in  std_logic_vector(15 downto 0)
        );
end Personality_ModuleV45;


architecture Behavioral of Personality_ModuleV45 is
  component risingEdgeCount
    generic (
      outWidth : integer range 0 to 32);
    port (
      sysClk   : in  std_logic;
      reset    : in  std_logic;
      signalIn : in  std_logic;
      count    : out unsigned(outWidth - 1 downto 0));
  end component;
  
  signal SignedTemp              : std_logic_vector(31 downto 0);
  signal control_reg             : std_logic_vector(15 downto 0);
  signal LocalWrt                : std_logic;
  signal PixCmdWrt               : std_logic;
  signal PixCmdWrtLock           : std_logic;
  signal PixCmdRead              : std_logic;

  signal startExp_reg : unsigned(15 downto 0);
  --signal rStartExp    : std_logic;
  --signal rrStartExp   : std_logic;
  signal clearStartExpCount : std_logic;

  signal syncInCnt_reg : unsigned(15 downto 0);
  signal clearSyncInCount : std_logic;

  signal syncOutCnt_reg : unsigned(15 downto 0);
  signal clearSyncOutCount : std_logic;

-- Specific register addresses mapped into the bit vector.
  signal CONTROL_CS    : std_logic;     -- decode_vector(0)
  signal STATUS_CS     : std_logic;     -- decode_vector(1)
  signal BOARD_CS      : std_logic;     -- decode_vector(2)
  signal FWVERDEC_CS   : std_logic;     -- decode_vector(3)
  signal CLOCK_CS      : std_logic;     -- decode_vector(4)
  signal SEQ_CTR_CS    : std_logic;     -- decode_vector(5)
  signal SEQ_CMD_CS    : std_logic;     -- decode_vector(6)
  signal SEQ_INT_CS    : std_logic;     -- decode_vector(7)
  signal SEQ_LPP_CS    : std_logic;     -- decode_vector(8)
  signal TST_CTR_CS    : std_logic;     -- decode_vector(9)
  signal SEQ_PAT_CS    : std_logic;     -- decode_vector(10)
  signal SEQ_PGM_CS    : std_logic;     -- decode_vector(11)
  signal PIX_FPGA_CS   : std_logic;     -- decode_vector(12)
  signal LED_REG_CS    : std_logic;     -- decode_vector(13)
  signal SN_DATA_CS    : std_logic;     -- decode_vector(14)
  signal TEMPDAT_CS    : std_logic;     -- decode_vector(15)
  signal SYNCDEL_CS    : std_logic;     -- decode_vector(16)
  signal PCBVersion_cs : std_logic;     -- decode_vector(17)
  signal FPGARLD_CS    : std_logic;     -- decode_vector(18)
  signal startExp_cs   : std_logic;     -- decode_vector(19)
  signal syncInCnt_cs   : std_logic;     
  signal syncOutCnt_cs   : std_logic;     
  signal clkDel_cs : std_logic;
  signal switches_cs : std_logic;

  signal cmd_out : std_logic;
  signal write_stb : std_logic;

  
  function csGenerator (
    constant regAddr  : std_logic_vector;
    signal   currAddr : std_logic_vector)
    return std_logic is
    variable tmp : std_logic := '1';
  begin
    for i in regAddr'range loop
      if regAddr(i) = '1' then
        tmp := tmp and currAddr(i);
      else
        tmp := tmp and not currAddr(i);
      end if;
    end loop;  -- i
    return tmp;
  end csGenerator;

begin

  -----------------------------------------------------------------------------
  -- Static assignments
  -----------------------------------------------------------------------------
  cmdRdy <= WRITE_STB and CMD_OUT;
  seqDataReq <= WRITE_STB and (not CMD_OUT);

------------------------------------------------------------------------
-- BOARD ADDRESS DECODE LOGIC
------------------------------------------------------------------------

-- Specific decode values for interface registers

-- external or internal selectors register at address 0xfff8
  switches_cs <= BOARD_SEL and csGenerator(ADDR_SWITCHES, ADDRESS);
  switches_sel <= switches_cs and WRITE;
  
-- Clock phase chip shares lines with sync delay chip. I've moved register
-- clkctrldec from pixel fpga to here. Now clkctrldec is called clkPhase
-- This address falls inside pixel range so I change how PIX_FPGA_CS is
-- generated.
  clkDel_cs <= BOARD_SEL and csGenerator(ADDR_CLKPHASE, ADDRESS);
  clkDel_sel <= clkDel_cs and WRITE;


-- Event register at 0xfff0 - provides strobes for common control functions
--  event_cs <= BOARD_SEL and csGenerator(ADDR_EVENTREG, ADDRESS);

-- Control Register select at 0xfffc
  CONTROL_CS <= BOARD_SEL and csGenerator(ADDR_SEQCTLREG, ADDRESS);

-- Status Register select at 0xfffd
  STATUS_CS <= BOARD_SEL and csGenerator(ADDR_SEQSTATUS, ADDRESS);

-- Board id Register select at 0xfffe
  BOARD_CS <= BOARD_SEL and csGenerator(ADDR_BOARDIDREG, ADDRESS);

-- Code id Register select at 0xffff
  FWVERDEC_CS <= BOARD_SEL and csGenerator(ADDR_FWVERDEC, ADDRESS);

-- Board specific values for logic implementation

-- Clock enable register selected at 0x0100
  CLOCK_CS    <= BOARD_SEL and csGenerator(ADDR_CLOCKEN, ADDRESS);
  CLOCKEN_SEL <= CLOCK_CS and WRITE;

-- Sequencer actual integration  time register selected at 0x0101
  SEQ_CTR_CS <= BOARD_SEL and csGenerator(ADDR_SEQACTINTTIM, ADDRESS);

-- Sequencer Command register selected at 0x0102
  SEQ_CMD_CS  <= BOARD_SEL and csGenerator(ADDR_SEQCMDREG, ADDRESS);
  SEQ_CMD_SEL <= SEQ_CMD_CS and WRITE;

-- Sequencer required integration time register selected at 0x0000 (32 bit write space)
  SEQ_INT_CS  <= BOARD_SEL and csGenerator(ADDR_SEQINTEGTIME, ADDRESS);
  SEQ_INT_SEL <= SEQ_INT_CS and WRITE;

-- Sequencer Loop registers selected from 0x110 to 0x11F
  SEQ_LPP_CS  <= BOARD_SEL and csGenerator(ADDR_RANGE_SEQLOOP, ADDRESS);
  SEQ_LPP_SEL <= SEQ_LPP_CS and WRITE;

-- Test counter register selected at 0x0001 (32 bit write space)
  TST_CTR_CS <= (BOARD_SEL and csGenerator(ADDR_COUNTREG, ADDRESS))
                or TST_CTR_REQ;
  TST_CTR_WRT <= TST_CTR_CS and WRITE;

-- Sequencer Pattern memory selected at 0x1000 to 0x1bff (Actual memory is 3 K words of 16 bit 0x1000 => 0x1BFF)
  SEQ_PAT_CS <= BOARD_SEL and csGenerator(ADDR_RANGE_SEQPATMEM, ADDRESS)
                and not (ADDRESS(11) and ADDRESS(10));
  SEQ_PAT_SEL <= SEQ_PAT_CS and WRITE;

-- Sequencer Program memory selected at 0x4000 to 0x43ff (Actual memory is 1 K words of 16 bit 0x2000 => 0x23FF)
  SEQ_PGM_CS <= BOARD_SEL and csGenerator(ADDR_RANGE_SEQPRGMEM, ADDRESS);

-- Pixel Fpga Command segment at 0x0200 => 0x3FF
  PIX_FPGA_CS <= ((BOARD_SEL and csGenerator(ADDR_RANGE_PIXCMD, ADDRESS))
                  and (not clkDel_cs))
                 or PixCmdWrt;
-- LED indicator register at address 0xfff9 - read / write
  LED_REG_CS  <= BOARD_SEL and csGenerator(ADDR_SEQ_LED, ADDRESS);
  LED_REG_SEL <= LED_REG_CS and WRITE;

-- Circuit board serial number at address 0xfffa - read only
  SN_DATA_CS <= BOARD_SEL and csGenerator(ADDR_BOARDSERIAL, ADDRESS);

-- Cicuit board temperature at address 0xfffb - read only (write triggers read of sensor) 
  TEMPDAT_CS  <= BOARD_SEL and csGenerator(ADDR_BOARDTEMP, ADDRESS);
--  TEMP_RD_STB <= TEMPDAT_CS and WRITE;

-- Sync delay register at 0x0601 - read / write
  SYNCDEL_CS  <= BOARD_SEL and csGenerator(ADDR_SYNC_DELAY, ADDRESS);
  SYNCDEL_SEL <= SYNCDEL_CS and WRITE;

-- PCB version register at 0x0600 - read
  PCBVersion_cs <= BOARD_SEL and csGenerator(ADDR_PCB_VERSION, ADDRESS);

-- Start Exposure stats register (how many times it had been fired
  startExp_cs <= BOARD_SEL and csGenerator(ADDR_STARTEXP, ADDRESS);
  clearStartExpCount <= reset or (WRITE and startExp_cs);

-- sync In rising edges stats register (how many times it had been fired
  syncInCnt_cs <= BOARD_SEL and csGenerator(ADDR_SYNCINCNT, ADDRESS);
  clearSyncInCount <= reset or (WRITE and syncInCnt_cs);

-- sync Out rising edges stats register (how many times it had been fired
  syncOutCnt_cs <= BOARD_SEL and csGenerator(ADDR_SYNCOUTCNT, ADDRESS);
  clearSyncOutCount <= reset or (WRITE and syncOutCnt_cs);

-- Reprogram fpga, address 0x0666 - read / write
  FPGARLD_CS <= BOARD_SEL and csGenerator(ADDR_REPROGRAM, ADDRESS);

  -- when writing any value to register 666 generate a pulse after 100ms
  program_n_delay : process (SYSCLK, reset)
    variable tickCount : integer range 0 to 65535 := 0;
    variable msCount   : integer range 0 to 127   := 0;
    variable run       : std_logic;
  begin  -- process program_n_delay
    if reset = '1' then                 -- asynchronous reset (active high)
      program_n <= '1';
      run       := '0';
      tickCount := 0;
      msCount   := 0;
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if ((FPGARLD_CS or FWVERDEC_CS) and write) = '1' then
        run       := '1';
        tickCount := 0;
        msCount   := 0;
        program_n <= '1';
      elsif run = '1' then
        if tickCount = 40000 then       -- 40000 ticks = 1 ms
          tickCount := 0;
          if msCount < 100 then         -- delay of 100ms
            msCount := msCount + 1;
          else
            program_n <= '0';
            msCount   := 0;
            run       := '0';
          end if;
        else
          tickCount := tickCount + 1;
        end if;
      else
        program_n <= '1';
        run       := '0';
      end if;
    end if;
  end process program_n_delay;




------------------------------------------------------------------------
-- IDENTITY CONSTANTS
-- Map the identity constants into this code module signals
-- Constants are defined inside constants package (constantsPkg.vhd)
------------------------------------------------------------------------
--  IDENT : IdentityModule port map (board_ident_out => board_ident, code_ident_out => code_ident);

------------------------------------------------------------------------
-- The control register
------------------------------------------------------------------------

  Cntrl_reg: process (sysClk, reset)
  begin  -- process Cntrl_reg
    if reset = '1' then                 -- asynchronous reset (active high)
      control_reg <= (4 => '1', others => '0');
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if CONTROL_CS = '1' and WRITE = '1' then
        control_reg <= DATA_IN(15 downto 0);
      end if;
    end if;
  end process Cntrl_reg;
  

  CONTROL_OUT  <= control_reg;
  TST_CTR_ENB  <= control_reg(7);

  -----------------------------------------------------------------------------
  -- Start exposure counter register
  -- It contains the number of times it has been fired
  -----------------------------------------------------------------------------
  startExpRisingEdgeCounter: risingEdgeCount
    generic map (
      outWidth => 16)
    port map (
      sysClk   => sysClk,
      reset    => clearStartExpCount,
      signalIn => exp2cntr,
      count    => startExp_reg);

  --startExpregProc : process (SYSCLK, reset)
  --begin  -- process startExpregProc
  --  if reset = '1' then                 -- asynchronous reset (active high)
  --    rStartExp    <= '0';
  --    rrStartExp   <= '0';
  --    startExp_reg <= (others => '0');
  --  elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
  --    rStartExp  <= start_exp;
  --    rrStartExp <= rStartExp;
  --    if rStartExp = '1' and rrStartExp = '0' then
  --      startExp_reg <= startExp_reg + 1;
  --    end if;
  --  end if;
  --end process startExpregProc;

  -----------------------------------------------------------------------------
  -- syncIn counter register
  -- It contains the number of times it has been fired
  -----------------------------------------------------------------------------
  risingEdgeCountSyncIn: risingEdgeCount
    generic map (
      outWidth => 16)
    port map (
      sysClk   => sysClk,
      reset    => clearSyncInCount,
      signalIn => crateSyncIn,
      count    => syncInCnt_reg);

  -----------------------------------------------------------------------------
  -- syncOut counter register
  -- It contains the number of times it has been fired
  -----------------------------------------------------------------------------
  risingEdgeCountSyncOut: risingEdgeCount
    generic map (
      outWidth => 16)
    port map (
      sysClk   => sysClk,
      reset    => clearSyncOutCount,
      signalIn => crateSyncOut,
      count    => syncOutCnt_reg);



------------------------------------------------------------------------
-- Sign extension for those data which return a signed result
------------------------------------------------------------------------

  SignedTemp <= (x"00000" & "00" & TEMP_DATA_IN) when (TEMP_DATA_IN(9) = '0')
                else (x"FFFFF" & "11" & TEMP_DATA_IN);

------------------------------------------------------------------------
-- DATA MULTIPLEXOR
-- If Status register is required, write in the port definition and
-- Remove the commented line to enable the status to be read accross
-- the bus. Add any data ports you require to this mux with relevent 
-- 'chip selects' derived from the address decode logic.
------------------------------------------------------------------------
  DataMuxMux : process (SYSCLK, reset)
  begin  -- process DataMuxMux
    if reset = '1' then                 -- asynchronous reset (active high)
      DATA_OUT <= x"12345678";
    elsif SYSCLK'event and SYSCLK = '1' then             -- rising clock edge
      if (ECHO_REQ = '1') then
        DATA_OUT <= ECHO_DATA_IN;
      else
        if CONTROL_CS = '1' then
          DATA_OUT <= x"0000" & control_reg;
        elsif STATUS_CS = '1' then
          DATA_OUT <= (x"0000" & STATUS_REG(15 downto 0));
        elsif BOARD_CS = '1' then
          DATA_OUT <= (x"0000" & BOARDID);
        elsif FWVERDEC_CS = '1' then
          DATA_OUT <= (x"0000" & FWVERDEC);
        elsif CLOCK_CS = '1' then
          DATA_OUT <= (x"0000" & CLOCKEN_REG(15 downto 0));
        elsif SEQ_CTR_CS = '1' then
          DATA_OUT <= SEQ_INT_CTR;
        elsif SEQ_CMD_CS = '1' then
          DATA_OUT <= (x"0000" & SEQ_CMD_REG(15 downto 0));
        elsif SEQ_INT_CS = '1' then
          DATA_OUT <= SEQ_INT_REG;
        elsif SEQ_LPP_CS = '1' then
          DATA_OUT <= (x"0000" & SEQ_LPP_REG(15 downto 0));
        elsif TST_CTR_CS = '1' then
          DATA_OUT <= TST_CTR_REG;
        elsif SEQ_PAT_CS = '1' then
          DATA_OUT <= (x"0000" & SEQ_PAT_MEM(15 downto 0));
        elsif SEQ_PGM_CS = '1' then
          DATA_OUT <= (x"0000" & SEQ_PGM_MEM(15 downto 0));
        elsif PIX_FPGA_CS = '1' then
          DATA_OUT <= ADDRESS & DATA_IN;
        elsif LED_REG_CS = '1' then
          DATA_OUT <= (x"0000000" & "00" & LED_DATA);
        elsif SN_DATA_CS = '1' then
          DATA_OUT <= SN_DATA(31 downto 0);
        elsif TEMPDAT_CS = '1' then
          DATA_OUT <= SignedTemp;
        elsif SYNCDEL_CS = '1' then
          DATA_OUT <= x"000000" & SYNCDEL_REG;
        elsif clkDel_cs = '1' then
          DATA_OUT <= x"000000" & clkDel_reg;
        elsif PCBVersion_cs = '1' then
          DATA_OUT <= x"0000"& PCB_VERSION;
        elsif FPGARLD_CS = '1' then
          DATA_OUT <= x"000BEA57";
        elsif startExp_cs = '1' then
          DATA_OUT <= x"0000" & std_logic_vector(startExp_reg);
        elsif syncInCnt_cs = '1' then
          DATA_OUT <= x"0000" & std_logic_vector(syncInCnt_reg);
        elsif syncOutCnt_cs = '1' then
          DATA_OUT <= x"0000" & std_logic_vector(syncOutCnt_reg);
        elsif switches_cs = '1' then
          DATA_OUT <= x"0000" & x"00" & "000" & switches_reg;
        else
          DATA_OUT <= x"0DEFA017";
        end if;
      end if;
    end if;
  end process DataMuxMux;

------------------------------------------------------------------------
-- FPGA INTERFACE WRITE STROBE DEVELOPMENT
------------------------------------------------------------------------
  LocalWrt <= (READ_STROBE and BOARD_SEL)
              or ECHO_REQ
              or TST_CTR_REQ
              or PixCmdWrt;

  DataMuxWriteStrobe : process (SYSCLK, reset)
  begin  -- process DataMuxWriteStrobe
    if reset = '1' then                 -- asynchronous reset (active high)
      WRITE_STB <= '0';
      pixSeqSyncOut  <= '0';     -- Not sure if it is active low or high
      CMD_OUT   <= '0';          -- same applies
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if (LocalWrt = '1') then
        WRITE_STB <= '1';
        if (TST_CTR_REQ = '1') then
          pixSeqSyncOut <= '0';  -- Sync should be false for any synthetic pixel
          -- data sent to pan.
          CMD_OUT  <= '0';
        elsif (PixCmdWrt = '1') then
          pixSeqSyncOut <= PixCmdRead;  -- Sync used to carry the command type if
          -- command being sent to Pix Fpga.
          CMD_OUT  <= '1';
        else
          pixSeqSyncOut <= '1';  -- Sync true for anything else sent to the pan.
          CMD_OUT  <= '0';
        end if;
      else
        WRITE_STB <= '0';
      end if;
    end if;
  end process DataMuxWriteStrobe;

  PixCmdWrtStrobe : process (SYSCLK, reset)
  begin  -- process PixCmdWrtStrobe
    if reset = '1' then                 -- asynchronous reset (active high)
      PixCmdWrt     <= '0';
      PixCmdWrtLock <= '0';
      PixCmdRead    <= '0';             -- No idea what we should put
    elsif SYSCLK'event and SYSCLK = '1' then  -- rising clock edge
      if PIX_FPGA_CS = '1' then  -- command passing over to the pix fpga.
        if PixCmdWrtLock = '0' then
          PixCmdRead    <= READ;        -- Capture the command type
          PixCmdWrt     <= '1';
          PixCmdWrtLock <= '1';
        else
          PixCmdWrt <= '0';
        end if;
      else
        PixCmdWrtLock <= '0';
        PixCmdWrt     <= '0';
      end if;
    end if;
  end process PixCmdWrtStrobe;

end Behavioral;
