------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   SEQUENCER BUS MULTIPLEXOR AND DRIVER
-- VERSION      :   2.1 
------------------------------------------------------------------------
-- FILE NAME    :   SEQUENCER_BUS_MUX.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   02/24/2003
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 4.2
------------------------------------------------------------------------
-- DESCRIPTION  : 
-- MULTIPLEXES THE DHE ORIGINATED DATA OR MPU SEQUENCER DATA
-- ONTO THE PHYSICAL SEQUENCER BUS. PROVIDES THE MPU SEQUENCER LOGIC WITH 
-- PRIORITY ON ACCESSING THE BUS. SINCE THE PHYSICAL BUS IS WRITE ONLY, THE
-- BUS IS PERMANENTLY ENABLED AND DRIVEN. 
--
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
--
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE   AUTHOR  VERSION  MODIFICATIONS
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- CLK              : ALL BUS UPDATES OCCUR ON RISING EDGE OF CLK
-- RST              : LOCAL RESET IN.
-- SEQ_BUS_REQ      : MPU SEQUENCER REQUEST FOR BUS USE
-- SEQ_BUS_ACK      : MPU SEQUENCER ACKNOWLEDGE FOR BUS USE (ALWAYS GRANTED ?)
-- SEQ_ADDRDATA_IN  : MPU SEQUENCER BUS GROUP INPUT
-- SEQ_DEV_ADDR_IN  : 
-- SEQ_MODE_IN      : 
-- SEQ_BOARD_SEL_IN :
-- DHE_BUS_REQ      : DHE DATA REQUEST FOR BUS (FROM FPGA INTERFACE).
-- DHE_BUS_ACK      : DHE DATA BUS GRANTED ACKNOWLEDGE (MAY WAIT FOR MPU SEQUENCER)
-- DHE_ADDRDATA_IN  : DHE DATA BUS GROUP INPUT
-- DHE_DEV_ADDR_IN  : 
-- DHE_MODE_IN      :
-- DHE_BOARD_SEL_IN : 
-- BUS_ADDRDATA_OUT : PHYSICAL BUS OUT
-- BUS_DEV_ADDR_OUT : 
-- BUS_MODE_OUT     : 
-- BUS_BOARD_SEL_OUT:
-- BOARD_ACK        : BUS CYCLE ACKNOWLEDGE BACK FROM SELECTED BOARD.
-- LOCAL_SELECT     : BOARD SELECT SIGNAL FOR LOCAL LOGIC ON MCB. 
-- LOCAL_ACK        :
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity Sequencer_Bus_Mux is
  port (CLK               : in  std_logic;
        RST               : in  std_logic;
        SEQ_BUS_REQ       : in  std_logic;
        SEQ_BUS_ACK       : out std_logic;
        SEQ_ADDRDATA_IN   : in  std_logic_vector(31 downto 0);
        SEQ_DEV_ADDR_IN   : in  std_logic_vector(4 downto 0);
        SEQ_MODE_IN       : in  std_logic_vector(1 downto 0);
        SEQ_BOARD_SEL_IN  : in  std_logic_vector(6 downto 0);
        DHE_BUS_REQ       : in  std_logic;
        DHE_BUS_ACK       : out std_logic;
        DHE_ADDRDATA_IN   : in  std_logic_vector(31 downto 0);
        DHE_DEV_ADDR_IN   : in  std_logic_vector(4 downto 0);
        DHE_MODE_IN       : in  std_logic_vector(1 downto 0);
        DHE_BOARD_SEL_IN  : in  std_logic_vector(7 downto 0);
        BUS_ADDRDATA_OUT  : out std_logic_vector(31 downto 0);
        BUS_DEV_ADDR_OUT  : out std_logic_vector(4 downto 0);
        BUS_MODE_OUT      : out std_logic_vector(1 downto 0);
        BUS_BOARD_SEL_OUT : out std_logic_vector(8 downto 2)

        );
end Sequencer_Bus_Mux;

architecture Behavioral of Sequencer_Bus_Mux is

  signal seq2bus : std_logic;

begin
------------------------------------------------------------------------
-- Determine who's going to get the bus (priority is RST, last user, then either SEQ or DHE).
------------------------------------------------------------------------
  seq_bus_mux : process (clk, RST)
  begin  -- process seq_bus_mux
    if RST = '1' then                   -- asynchronous reset (active high)
      seq2bus           <= '0';
      BUS_BOARD_SEL_OUT <= (others => '1');
      BUS_ADDRDATA_OUT  <= (others => '0');
      BUS_DEV_ADDR_OUT  <= (others => '0');
      BUS_MODE_OUT      <= (others => '0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if (seq2bus = '1') then
        if (SEQ_BUS_REQ = '1') then
          BUS_BOARD_SEL_OUT <= not SEQ_BOARD_SEL_IN(6 downto 0);
          BUS_ADDRDATA_OUT  <= SEQ_ADDRDATA_IN;
          BUS_DEV_ADDR_OUT  <= SEQ_DEV_ADDR_IN;
          BUS_MODE_OUT      <= SEQ_MODE_IN;
          seq2bus           <= '1';
        elsif (DHE_BUS_REQ = '1') then
          BUS_BOARD_SEL_OUT <= not DHE_BOARD_SEL_IN(7 downto 1);
          BUS_ADDRDATA_OUT  <= DHE_ADDRDATA_IN;
          BUS_DEV_ADDR_OUT  <= DHE_DEV_ADDR_IN;
          BUS_MODE_OUT      <= DHE_MODE_IN;
          seq2bus           <= '0';
        else
          BUS_BOARD_SEL_OUT <= (others => '1');
        end if;
      else
        if (DHE_BUS_REQ = '1') then
          BUS_BOARD_SEL_OUT <= not DHE_BOARD_SEL_IN(7 downto 1);
          BUS_ADDRDATA_OUT  <= DHE_ADDRDATA_IN;
          BUS_DEV_ADDR_OUT  <= DHE_DEV_ADDR_IN;
          BUS_MODE_OUT      <= DHE_MODE_IN;
          seq2bus           <= '0';
        elsif (SEQ_BUS_REQ = '1') then
          BUS_BOARD_SEL_OUT <= not SEQ_BOARD_SEL_IN(6 downto 0);
          BUS_ADDRDATA_OUT  <= SEQ_ADDRDATA_IN;
          BUS_DEV_ADDR_OUT  <= SEQ_DEV_ADDR_IN;
          BUS_MODE_OUT      <= SEQ_MODE_IN;
          seq2bus           <= '1';
        else
          BUS_BOARD_SEL_OUT <= (others => '1');
        end if;
      end if;
    end if;
  end process seq_bus_mux;

  DHE_BUS_ACK <= '0' when (seq2bus = '1') else '1';

  SEQ_BUS_ACK <= '1' when (seq2bus = '1') else '0';

end Behavioral;
