------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   MASTER CONTROL BOARD
-- DEVICE ID    :   MCB SEQUENCER FPGA U22
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   TEMPERATURE SENSOR READ LOGIC
-- VERSION      :   4.2 
------------------------------------------------------------------------
-- FILE NAME    :   TEMPSENSOR.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   05/19/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- INTERFACES A AD7814 3 WIRE TEMP SENSOR.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE      AUTHOR  VERSION  MODIFICATIONS
-- 05/19/2004  PCM      4.2   ORIGINAL VERSION
-- 31/03/2009  OBB            
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK    : SYSTEM CLOCK
-- READ_TEMP : TRIGGERS A READ FROM THE TEMPERATURE SENSOR
-- TEMP_DATA : TEMPERATURE DATA OUT TO MUX
-- TEMP_DIN  : SERIAL DATA INTO SENSOR
-- TEMP_DOUT : SERIAL DATA OUT FROM SENSOR
-- TEMP_SCLK : SERIAL SENSOR DATA CLOCK
-- TEMP_CS   : Serial chipselect
------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity TempSensor is
  port (
    SYSCLK    : in  std_logic;
    RESET     : in  std_logic;
    TEMP_DATA : out std_logic_vector(9 downto 0);
    TEMP_DIN  : out std_logic;
    TEMP_DOUT : in  std_logic;
    TEMP_CS   : out std_logic;
    TEMP_SCLK : out std_logic
    );
end TempSensor;

architecture Behavioral of TempSensor is

  signal reset_n : std_logic;

  -- Number of bits in a serial frame (16)
  constant FRAMEBITS : std_logic_vector(4 downto 0) := "10000";

  -- Number of clock (sysClk) cycles between adquisitions (500ms)
  constant WAITCLOCKS : std_logic_vector(27 downto 0) := x"1312D00";

  -- SCLK half period length in sysClk periods
  constant SCLKCLKDIV : integer := 4;

  signal currFrameBit : std_logic_vector(4 downto 0);
  signal idleCount    : std_logic_vector(27 downto 0);

--  signal serDI : std_logic_vector(15 downto 0);
  signal serDO : std_logic_vector(14 downto 0);

  type tempStates is (tIdle, tAccess, tStore);
  signal t_CurrSt, t_NextSt : tempStates;

  signal sclkEn   : std_logic;          -- Serial clock enable
  signal sclk     : std_logic;          -- Serial clock
  signal sclk_reg : std_logic;  -- Registered serial clock (Used to detect sclk
-- edges) 
begin

  reset_n <= not reset;

  -----------------------------------------------------------------------------
  -- FSM implementation
  -----------------------------------------------------------------------------
  -- Current state assignment
  proc_currState : process (sysClk, reset_n)
  begin  -- process proc_currState
    if reset_n = '0' then               -- asynchronous reset (active low)
      t_CurrSt <= tIdle;
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      t_CurrSt <= t_NextSt;
    end if;
  end process proc_currState;

  --Next state generation
  t_NextSt <= tIdle when reset_n = '0' else
              tAccess when t_CurrSt = tIdle and idleCount = WAITCLOCKS else
              tStore when (t_CurrSt = tAccess
                            and currFrameBit = x"f"
                            and sclk = '1'
                            and sclk_reg = '0') else
              tAccess when t_CurrSt = tAccess else
              tIdle;


  -----------------------------------------------------------------------------
  -- Control signals
  -----------------------------------------------------------------------------
  -- serial clock enable
  sclkEn <= '1' when t_CurrSt = tAccess else
            '0';

  -- Current frame bit
  proc_currFrameBit : process (sysClk, reset_n)
    variable enable : std_logic;
  begin  -- process proc_currFrameBit
    if reset_n = '0' then               -- asynchronous reset (active low)
      currFrameBit <= (others => '0');
      enable       := '0';
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if t_CurrSt = tAccess then
        if sclk = '0' and sclk_reg = '1' then
          if enable = '1' then
            currFrameBit <= currFrameBit + 1;
          else
            enable := '1';
          end if;
        end if;
      else
        enable       := '0';
        currFrameBit <= (others => '0');
      end if;
    end if;
  end process proc_currFrameBit;

  -- Sysclk rising edge counter for Idle state
  proc_idleCount : process (sysClk, reset_n)
  begin  -- process proc_idleCount
    if reset_n = '0' then               -- asynchronous reset (active low)
      idleCount <= (others => '0');
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if t_CurrSt = tIdle then
        idleCount <= idleCount + 1;
      else
        idleCount <= (others => '0');
      end if;
    end if;
  end process proc_idleCount;

  -----------------------------------------------------------------------------
  -- temperature ouput register (TEMP_DATA)
  -----------------------------------------------------------------------------
  proc_TEMP_DATA : process (sysClk, reset_n)
  begin  -- process proc_TEMP_DATA
    if reset_n = '0' then               -- asynchronous reset (active low)
      TEMP_DATA <= (others => '0');
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if t_CurrSt = tStore then
        TEMP_DATA <= serDO(14 downto 5);
      end if;
    end if;
  end process proc_TEMP_DATA;

  -----------------------------------------------------------------------------
  -- Deserializer register
  -----------------------------------------------------------------------------
  proc_Deserializer : process (sysClk, reset_n)
  begin  -- process proc_Deserializer
    if reset_n = '0' then               -- asynchronous reset (active low)
      serDO <= (others => '0');
    elsif sysClk'event and sysClk = '1' then      -- rising clock edge
      if t_CurrSt = tIdle then
        serDO <= (others => '0');
      elsif t_CurrSt = tAccess then
        if sclk = '1' and sclk_reg = '0' then     -- sclk rising edge
          serDO(0) <= TEMP_DOUT;
        elsif sclk = '0' and sclk_reg = '1' then  -- sclk falling edge
          serDO(14 downto 1) <= serDO(13 downto 0);
        end if;
      end if;
    end if;
  end process proc_Deserializer;

  -----------------------------------------------------------------------------
  -- Serial output (TEMP_DIN, input to temp sensor, output from FPGA)
  -----------------------------------------------------------------------------
  -- As I won't set it on power down mode it always outputs '0'
  TEMP_DIN <= '0';

  -----------------------------------------------------------------------------
  -- Serial Chip Select
  -----------------------------------------------------------------------------
  TEMP_CS <= '0' when t_CurrSt = tAccess else
             '1';

  -----------------------------------------------------------------------------
  -- Serial clock generation
  -----------------------------------------------------------------------------
  proc_sclk : process (sysClk, reset_n)
    variable clkDivider : integer range 0 to SCLKCLKDIV - 1 := 0;
  begin  -- process proc_sclk
    if reset_n = '0' then               -- asynchronous reset (active low)
      clkDivider := 0;
      sclk       <= '1';
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if sclkEn = '0' then
        clkDivider := (SCLKCLKDIV - 1);
        sclk       <= '1';
      else
        if clkDivider = (SCLKCLKDIV - 1) then
          clkDivider := 0;
          sclk       <= not sclk;
        else
          clkDivider := clkDivider + 1;
        end if;
      end if;
    end if;
  end process proc_sclk;

  TEMP_SCLK <= sclk;


  -----------------------------------------------------------------------------
  -- Serial clock registered
  -----------------------------------------------------------------------------
  -- Used to track edges of sclk avoiding two clock trees

  proc_sclk_reg : process (sysClk, reset_n)
  begin  -- process proc_sclk_reg
    if reset_n = '0' then               -- asynchronous reset (active low)
      sclk_reg <= '0';
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      sclk_reg <= sclk;
    end if;
  end process proc_sclk_reg;



end Behavioral;
