-------------------------------------------------------------------------------
-- Title      : delayChips
-- Project    : 
-------------------------------------------------------------------------------
-- File       : delayChips.vhd
-- Author     :   <otger@OTGER-PC>
-- Company    : 
-- Created    : 2009-12-23
-- Last update: 2009-12-29
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: component to control to delay chips inside the mcb sequencer.
-- There are two chips which shares clk and data signals. Each one has its own
-- enable line and its own register which holds which value has to be
-- programmed to each chip
-------------------------------------------------------------------------------
-- Copyright (c) 2009 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2009-12-23  1.0      otger   Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.packageSeq.all;


entity delayChips is
  
  port (
    sysClk           : in  std_logic;
    reset            : in  std_logic;   -- active high reset
    syncWrite        : in  std_logic;   -- write new sync value
    clkWrite         : in  std_logic;   -- write new clk value
    dataIn           : in  std_logic_vector(7 downto 0);
    delayChipClk     : out std_logic;
    delayChipData    : out std_logic;
    syncDelayChipEn  : out std_logic;
    clkDelayChipEn   : out std_logic;
    syncDelayCurrVal : out std_logic_vector(7 downto 0);
    clkDelayCurrVal  : out std_logic_vector(7 downto 0));

end delayChips;

architecture twin of delayChips is

  signal syncFire : std_logic;
  signal clkFire  : std_logic;

  signal syncValReg : std_logic_vector(7 downto 0);
  signal clkValReg  : std_logic_vector(7 downto 0);

  signal syncDelChipClk : std_logic;
  signal syncDelChipD   : std_logic;
  signal syncDelChipE   : std_logic;

  signal clkDelChipClk : std_logic;
  signal clkDelChipD   : std_logic;
  signal clkDelChipE   : std_logic;

  signal int_clkDelayCurrVal : std_logic_vector(7 downto 0);
  signal int_syncDelayCurrVal : std_logic_vector(7 downto 0);
  
  
begin  -- twin

  clkDelayCurrVal <= int_clkDelayCurrVal;
  syncDelayCurrVal <= int_syncDelayCurrVal;

  syncDelayChip : delayChipCtrl
    port map (
      sysclk      => sysclk,
      reset       => reset,
      write       => syncFire,
      delayDatIn  => syncValReg,
      delayDatOut => int_syncDelayCurrVal,
      delChipClk  => syncDelChipClk,
      delChipD    => syncDelChipD,
      delChipE    => syncDelChipE);

  clkDelayChip : delayChipCtrl
    port map (
      sysclk      => sysclk,
      reset       => reset,
      write       => clkFire,
      delayDatIn  => clkValReg,
      delayDatOut => int_clkDelayCurrVal,
      delChipClk  => clkDelChipClk,
      delChipD    => clkDelChipD,
      delChipE    => clkDelChipE);

  -----------------------------------------------------------------------------
  -- Registers to hold values to write to chips
  -----------------------------------------------------------------------------
  syncValue : process (sysClk, reset)
  begin  -- process syncValue
    if reset = '1' then                 -- asynchronous reset (active high)
      syncValReg <= (others => '0');
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if syncWrite = '1' then
        syncValReg <= dataIn;
      end if;
    end if;
  end process syncValue;

  clkValue : process (sysClk, reset)
  begin  -- process clkValue
    if reset = '1' then                 -- aclkhronous reset (active high)
      clkValReg <= (others => '0');
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if clkWrite = '1' then
        clkValReg <= dataIn;
      end if;
    end if;
  end process clkValue;

  -----------------------------------------------------------------------------
  -- Selector with priority
  -----------------------------------------------------------------------------
  selector : process (sysClk, reset)
  begin  -- process selector
    if reset = '1' then                 -- asynchronous reset (active low)
      syncFire <= '0';
      clkFire  <= '0';
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if clkDelChipE = '1' or syncDelChipE = '1' then
        syncFire <= '0';
        clkFire <= '0';
      elsif clkValReg /= int_clkDelayCurrVal then
        clkFire <= '1';
      elsif syncValReg /= int_syncDelayCurrVal then
        syncFire <= '1';
      else
        syncFire <= '0';
        clkFire  <= '0';
      end if;
    end if;
  end process selector;

  clkDelayChipEn  <= clkDelChipE;
  syncDelayChipEn <= syncDelChipE when clkDelChipE = '0'
                     else '0';

  delayChipData <= clkDelChipD when clkDelChipE = '1' else
                   syncDelChipD when syncDelChipE = '1' else
                   '0';
  delayChipClk <= clkDelChipClk when clkDelChipE = '1' else
                  syncDelChipClk when syncDelChipE = '1' else
                  '0';

  
end twin;
