-------------------------------------------------------------------------------
-- Title      : led control sequencer
-- Project    : 
-------------------------------------------------------------------------------
-- File       : ledctrlSeq.vhd
-- Author     :   <otger@OTGER-PC>
-- Company    : 
-- Created    : 2010-12-15
-- Last update: 2010-12-15
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: Simple led control module for MCB front panel leds.
-- HW version 1.2 of MCB has two front panel leds: isMaster and sequencer
-- isMaster led shows is the MCB is set as Master
-- sequencer led shows when the sequencer is running

-- For both sources, when ledSource becomes true, led flashes for 25ms at least
-- If signal stays true for a longer time, led stays on all the time
-------------------------------------------------------------------------------
-- Copyright (c) 2010 
-------------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- sysclk       : System Clock
-- reset        : reset signal for registers
-- wrt          : write signal to load dataIn into ledEnableRegister
-- dataIn       : input data to write to ledEnableRegister
-- dataOut      : ledEnableRegister current value
-- isMasterSrc  : source for the isMaster led
-- seqRunSrc    : source for the sequencer running led
-- isMasterLed  : signal to feed isMaster Led
-- seqRunLed    : signal to feed sequencer running led
------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2010-12-15  1.0      otger   Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity LedCtrlSeq is
  port (
    sysClk      : in  std_logic;
    reset       : in  std_logic;
    wrt         : in  std_logic;
    dataIn      : in  std_logic_vector(1 downto 0);
    dataOut     : out std_logic_vector(1 downto 0);
    isMasterSrc : in  std_logic;
    seqRunSrc   : in  std_logic;
    isMasterLed : out std_logic;
    seqRunLed   : out std_logic
    );
end LedCtrlSeq;

architecture Behavioral of LedCtrlSeq is

  -- 25 ms pulse width.
  constant LED_PERIOD : std_logic_vector(19 downto 0) := x"F4240";

  signal ledEnReg   : std_logic_vector(1 downto 0);
  signal srcReg     : std_logic_vector(1 downto 0);
  signal risingEdge : std_logic_vector(1 downto 0);
  signal ledFlash   : std_logic_vector(1 downto 0);
  signal firstRun   : std_logic_vector(1 downto 0);

  type nineteen is array (0 to 1) of std_logic_vector(19 downto 0);
  signal counter : nineteen;
begin

------------------------------------------------------------------------
-- Ya' gotta catch the data value to set the the bits baby!
------------------------------------------------------------------------
  enableRegister : process (sysClk, reset)
  begin  -- process enableRegister
    if reset = '1' then                 -- asynchronous reset (active high)
      ledEnReg <= "00";
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if wrt = '1' then
        ledEnReg(1 downto 0) <= dataIn(1 downto 0);
      end if;
    end if;
  end process enableRegister;

  registeringSources : process (sysClk, reset)
  begin  -- process registeringSources
    if reset = '1' then                 -- asynchronous reset (active high)
      srcReg <= "11";   -- To avoid detecting false rising up after reset
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      srcReg(0) <= isMasterSrc;
      srcReg(1) <= seqRunSrc;
    end if;
  end process registeringSources;

  risingEdge(0) <= isMasterSrc xor srcReg(0);
  risingEdge(1) <= seqRunSrc xor srcReg(1);

  flashGenerator : for i in 0 to 1 generate
    counterReg : process (sysClk, reset)
    begin  -- process counterReg
      if reset = '1' then               -- asynchronous reset (active high)
        counter(i)  <= (others => '0');  -- This triggers a flash after reset
        ledFlash(i) <= '0';
        firstRun(i) <= '1';
      elsif sysClk'event and sysClk = '1' then  -- rising clock edge
        if risingEdge(i) = '1' then
          counter(i)  <= (others => '0');
          ledFlash(i) <= '1';
        else
          if counter(i) < LED_PERIOD then
            counter(i) <= counter(i) + 1;
          else
            ledFlash(i) <= '0';
            firstRun(i) <= '0';
          end if;
        end if;
      end if;
    end process counterReg;
  end generate flashGenerator;

  isMasterLed <= (ledFlash(0) or isMasterSrc) and
                 (ledEnReg(0) or firstRun(0));
  seqRunLed   <= (ledFlash(1) or seqRunSrc) and 
                 (ledEnReg(1) or firstRun(1));

  dataOut <= LedEnReg;

end Behavioral;
