-- serno_interface.vhd
-- read the serial number from a dallas DS2401 "one wire" chip
-- Jamieson Olsen <jamieson@fnal.gov>

-- Assumes that the DS2401 is the only device on the sn_io line and
-- that there is a pullup resistor (enabled inside the FPGA or
-- soldered to the board externally). All reads and writes are
-- LSb first.

-- RESET pulse: low for 500us, then release and wait 15us
-- write 1: low for 10us, then release and wait 90us
-- write 0: low for 80us, then release and wait 20us
-- read: low for 10us, release, wait 5us, sample, wait 85us

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

entity serno_interface is
  port (
    clock   : in    std_logic;          -- 40MHz system clock
    reset   : in    std_logic;          -- active HIGH reset
    sn_data : out   std_logic_vector(31 downto 0);
--    sn_crc_ok : out   std_logic;
    sn_io   : inout std_logic
    );
end serno_interface;

architecture serno_interface_arch of serno_interface is

  -- constants

  constant read_cmd : std_logic_vector(7 downto 0) := X"33";

  -- time constants (edit these)

  constant sys_mhz         : integer := 40;   -- clock freq in MHz
  constant rst_time        : integer := 500;  -- master reset pulse
  constant rst_rec_time    : integer := 15;   -- recovery after reset
  constant write1_time     : integer := 10;   -- time slot for reads and writes
  constant write1_rec_time : integer := 90;   -- time slot for reads and writes
  constant write0_time     : integer := 75;   -- time slot for reads and writes
  constant write0_rec_time : integer := 25;   -- time slot for reads and writes
  constant read_time       : integer := 10;   -- pull down to read a bit
  constant sample_time     : integer := 5;    -- then wait awhile and sample
  constant read_rec_time   : integer := 85;   -- recover time in between bits

  -- convert time constants to timer counts (assume 40MHz clock)  DON'T edit this

  constant v_rst        : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(rst_time*sys_mhz, 15));
  constant v_rst_rec    : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(rst_rec_time*sys_mhz, 15));
  constant v_write1     : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(write1_time*sys_mhz, 15));
  constant v_write1_rec : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(write1_rec_time*sys_mhz, 15));
  constant v_write0     : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(write0_time*sys_mhz, 15));
  constant v_write0_rec : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(write0_rec_time*sys_mhz, 15));
  constant v_read       : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(read_time*sys_mhz, 15));
  constant v_sample     : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(sample_time*sys_mhz, 15));
  constant v_read_rec   : std_logic_vector(14 downto 0) := std_logic_vector(to_unsigned(read_rec_time*sys_mhz, 15));

  -- define registers...

  signal go_reg     : std_logic;
  signal data_reg   : std_logic_vector(47 downto 0);
  signal timer_reg  : std_logic_vector(14 downto 0);
  signal count_reg  : std_logic_vector(5 downto 0);
  signal cmd_sr_reg : std_logic_vector(7 downto 0);
--  signal crc_reg    : std_logic_vector(7 downto 0);

  type state_type is (rst, wait4read, rstpulse, rst_rec, wait4pres, pres_rec1, pres_rec2, write_bit, write_rec, read_start, read_sample, read_rec);
  signal state : state_type;

  -- declare components...

--  component pullup
--    port (o : out std_logic);
--  end component;

begin

  -- combinatorial stuff...

  sn_data <= data_reg(31 downto 0) when (state = wait4read) else
             (others => '0');

  --sn_busy <= '0' when (state = wait4read) else '1';

--  sn_crc_ok <= '1' when (crc_reg = X"00" and state = wait4read) else '0';

  -- sequential stuff...

  fsm : process(clock)
  begin
    if rising_edge(clock) then

      if (reset = '1') then
        state     <= rst;
        timer_reg <= (others => '0');
        count_reg <= (others => '0');
        data_reg  <= (others => '0');
--        crc_reg   <= X"00";
        go_reg    <= '1';               -- trigger go after 

      else

        -- go_reg <= sn_update;

        case (state) is

          when rst =>
            sn_io <= 'Z';
            state <= wait4read;

          when wait4read =>             -- wait here for the user to hit update
            sn_io <= 'Z';
            if (go_reg = '1') then
              state     <= rstpulse;
              timer_reg <= (others => '0');
              go_reg    <= '0';
            else
              state <= wait4read;
            end if;

          when rstpulse =>  -- send a reset pulse to the slave device
            sn_io <= '0';
            if (timer_reg = v_rst) then
              state     <= rst_rec;
              timer_reg <= (others => '0');
            else
              state     <= rstpulse;
              timer_reg <= timer_reg + 1;
            end if;

          when rst_rec =>               -- sent the reset pulse, now recover
            sn_io <= 'Z';
            if (timer_reg = v_rst_rec) then
              state     <= wait4pres;
              timer_reg <= (others => '0');
            else
              state     <= rst_rec;
              timer_reg <= timer_reg + 1;
            end if;

          when wait4pres =>  -- wait here until the slave responds with a presence pulse
            sn_io <= 'Z';
            if (sn_io = '0') then
              state     <= pres_rec1;
              timer_reg <= (others => '0');
            else
              state     <= wait4pres;
              timer_reg <= timer_reg + 1;
            end if;

          when pres_rec1 =>  -- slave is holding the line low with presence pulse, wait here
            sn_io     <= 'Z';
            timer_reg <= (others => '0');
            if (sn_io = '0') then
              state <= pres_rec1;
            else
              state <= pres_rec2;
            end if;

          when pres_rec2 =>  -- slave just released the presense pulse, recover again
            sn_io <= 'Z';
            if (timer_reg = v_rst_rec) then
              state      <= write_bit;
              timer_reg  <= (others => '0');
              count_reg  <= "000111";   -- command is 8 bits
              cmd_sr_reg <= read_cmd;   -- read ROM opcode
            else
              state     <= pres_rec2;
              timer_reg <= timer_reg + 1;
            end if;

          when write_bit =>  -- write a 0 or 1 depending on LSb of cmd_sr_reg
            sn_io <= '0';
            if ((timer_reg = v_write0 and cmd_sr_reg(0) = '0') or
                (timer_reg = v_write1 and cmd_sr_reg(0) = '1')) then
              timer_reg <= (others => '0');
              state     <= write_rec;
            else
              state     <= write_bit;
              timer_reg <= timer_reg + 1;
            end if;

          when write_rec =>
            sn_io <= 'Z';
            if (((timer_reg = v_write0_rec and cmd_sr_reg(0) = '0') or
                 (timer_reg = v_write1_rec and cmd_sr_reg(0) = '1')) and
                count_reg = "000000") then  -- we're done with the write
              state     <= read_start;
              timer_reg <= (others => '0');
--              crc_reg   <= (others => '0');
              count_reg <= "111111";  -- read 64 bits (8 ID + 48 serno + 8 CRC)

            elsif (((timer_reg = v_write0_rec and cmd_sr_reg(0) = '0') or
                    (timer_reg = v_write1_rec and cmd_sr_reg(0) = '1')) and
                   count_reg > "000000") then  -- shift and transmit again
              state                  <= write_bit;
              timer_reg              <= (others => '0');
              count_reg              <= count_reg - 1;
              cmd_sr_reg(6 downto 0) <= cmd_sr_reg(7 downto 1);  -- shift RIGHT
              cmd_sr_reg(7)          <= '0';
            else
              state     <= write_rec;
              timer_reg <= timer_reg + 1;
            end if;

          when read_start =>            -- drive low to start to read a bit
            sn_io <= '0';
            if (timer_reg = v_read) then
              state     <= read_sample;
              timer_reg <= (others => '0');
            else
              state     <= read_start;
              timer_reg <= timer_reg + 1;
            end if;

          when read_sample =>  -- let go and wait for a few microseconds before sampling
            sn_io <= 'Z';
            if (timer_reg = v_sample) then
              state     <= read_rec;
              timer_reg <= (others => '0');

              if (count_reg < "111000" and count_reg > "001000") then  -- this is the serial number data, keep it.
                data_reg(46 downto 0) <= data_reg(47 downto 1);  -- note data coming in LSb first!

                if (sn_io = '0') then
                  data_reg(47) <= '0';
                else
                  data_reg(47) <= '1';
                end if;

              end if;

              --if (sn_io = '0') then     -- do the CRC calculation here
              --  crc_reg(0) <= crc_reg(7) xor '0';
              --  crc_reg(1) <= crc_reg(0);
              --  crc_reg(2) <= crc_reg(1);
              --  crc_reg(3) <= crc_reg(2);
              --  crc_reg(4) <= (crc_reg(7) xor '0') xor crc_reg(3);
              --  crc_reg(5) <= (crc_reg(7) xor '0') xor crc_reg(4);
              --  crc_reg(6) <= crc_reg(5);
              --  crc_reg(7) <= crc_reg(6);
              --else
              --  crc_reg(0) <= crc_reg(7) xor '1';
              --  crc_reg(1) <= crc_reg(0);
              --  crc_reg(2) <= crc_reg(1);
              --  crc_reg(3) <= crc_reg(2);
              --  crc_reg(4) <= (crc_reg(7) xor '1') xor crc_reg(3);
              --  crc_reg(5) <= (crc_reg(7) xor '1') xor crc_reg(4);
              --  crc_reg(6) <= crc_reg(5);
              --  crc_reg(7) <= crc_reg(6);
              --end if;

            else
              state     <= read_sample;
              timer_reg <= timer_reg + 1;
            end if;

          when read_rec =>              -- recover after sampling the data bit
            sn_io <= 'Z';
            if (timer_reg = v_read_rec) then
              if (count_reg = "000000") then  -- we're done.
                state <= wait4read;
              else
                count_reg <= count_reg - 1;   -- get the next bit
                state     <= read_start;
                timer_reg <= (others => '0');
              end if;
            else
              state     <= read_rec;
              timer_reg <= timer_reg + 1;
            end if;

          when others =>
            state <= rst;

        end case;
      end if;
    end if;
  end process;

  -- instantiate components...

  pullup_res : pullup
    port map(o => sn_io);

end serno_interface_arch;
