-------------------------------------------------------------------------------
-- Title      : switchesSelec
-- Project    : 
-------------------------------------------------------------------------------
-- File       : switchesSelec.vhd
-- Author     :   <otger@OTGER-PC>
-- Company    : 
-- Created    : 2009-12-28
-- Last update: 2009-12-28
-- Platform   : 
-- Standard   : VHDL'87
-------------------------------------------------------------------------------
-- Description: Register to select behavior of clk and sync selectors. Instead
-- of jumpers MCB 1.2 has switches.
-------------------------------------------------------------------------------
-- Copyright (c) 2009 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2009-12-28  1.0      otger   Created
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- What's this module?
-------------------------------------------------------------------------------
-- There are two digital switches on the MCB 1.2 to select if we want internal
-- or external clock and sync signals.
-- When clkSwitchSel is '0', internal source is selected and when is '1',
-- external source is selected as output. Same applies for syncSwitchSel.
-- By default this module autoselect if mcb runs with internal or external
-- signals. This is done based on isMaster signal (is a copy of control
-- register bit 4, which is set when MCB is Master and unset when MCB is slave)
-- By default, MCB boots as master (control_reg(4) == '1') and internal signals
-- are selected.
-- If bit 0 of this register is unset then, One must select if we want to use
-- internal or external signals using bits 1 (clk selector) and 2 (sync
-- selector). On bits 3 (clk) and 4 (sync), one can read current value.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity switchesSelec is
  
  port (
    sysClk        : in  std_logic;
    reset         : in  std_logic;      -- active high reset
    writeReg      : in  std_logic;
    dataIn        : in  std_logic_vector(2 downto 0);
    dataOut       : out std_logic_vector(4 downto 0);
    clkSwitchSel  : out std_logic;
    syncSwitchSel : out std_logic;
    isMaster      : in  std_logic);

end switchesSelec;

architecture mine of switchesSelec is

  signal regData     : std_logic_vector(4 downto 0);
  signal autoSelect  : std_logic;
  signal setClkSel   : std_logic;
  signal setSyncSel  : std_logic;
  signal currClkSel  : std_logic;
  signal currSyncSel : std_logic;
  
begin  -- mine

  -- when setClkSel is set or board is Master (isMaster = 1) currClkSel must be
  -- 0, to select internal clock. When setClkSel is unset, currClkSel must be
  -- '1' to use external clock
  clkSwitchSel  <= not currClkSel;  
  syncSwitchSel <= not currSyncSel;    
  dataOut       <= regData;

  -----------------------------------------------------------------------------
  -- Register Data
  -----------------------------------------------------------------------------
  regData(0) <= autoSelect;             -- RW
  regData(1) <= setClkSel;              -- RW
  regData(2) <= setSyncSel;             -- RW
  regData(3) <= currClkSel;             -- R only
  regData(4) <= currSyncSel;            -- R only

  regDataProc : process (sysClk, reset)
  begin  -- process regDataProc
    if reset = '1' then                 -- asynchronous reset (active high)
      autoSelect <= '1';
      setClkSel  <= '1';
      setSyncSel <= '1';
    elsif sysClk'event and sysClk = '1' then  -- rising clock edge
      if writeReg = '1' then
        autoSelect <= dataIn(0);
        setClkSel  <= dataIn(1);
        setSyncSel <= dataIn(2);
      end if;
    end if;
  end process regDataProc;

  -----------------------------------------------------------------------------
  -- Calculate current select values
  -----------------------------------------------------------------------------

  currentSelValues : process (autoSelect, isMaster, setClkSel, setSyncSel)
  begin  -- process currClkSel
    if autoSelect = '1' then
      currClkSel  <= isMaster;
      currSyncSel <= isMaster;
    else
      currClkSel  <= setClkSel;
      currSyncSel <= setSyncSel;        -- same applies
    end if;
  end process currentSelValues;

end mine;
